<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_backup_dbTable extends we_backup_contentBase{
	public const BACKUP_XML_TAG = 'we:dbtable';

	/** @var we_database_base */
	protected $db;

	/** @var string */
	public $table;

	/** @var string */
	protected $defaultTableName;

	/** @var string[] */
	protected $elements = [];

	public function __construct(string $table, bool $force_columns = false){
		$this->db = new DB_WE();
		$this->table = $table;

		$this->attribute_slots = [
			'name' => stripTblPrefix($table)
		];

		/* $update_table = true;
		  $lower = strtolower($table);
		  if(!$force_columns){
		  if(substr($lower, 0, 10) === strtolower(stripTblPrefix(OBJECT_X_TABLE)) ||
		  $lower === strtolower(stripTblPrefix(CUSTOMER_TABLE))
		  ){
		  $update_table = false;
		  }
		  }

		  if($update_table){
		  $this->getColumns();
		  } */
	}

	public function getLogString(string $prefix = ''): string{
		return $prefix . $this->table;
	}

	public function getColumns(): void{
		if($this->db->isTabExist($this->table)){
			$zw = $this->db->getTableCreateArray($this->table);
			$zw[0] = str_replace($this->table, stripTblPrefix($this->table), $zw[0]);
			$this->elements = $zw;
		}
	}

	public function getDefaultTableName(): string{
		return $this->defaultTableName;
	}

	/**
	 *
	 * @return array{name:string,create:string[]}
	 */
	protected function getBackupData(): array{
		$this->getColumns();
		$this->defaultTableName = we_backup_util::getDefaultTableName($this->table);
		return [
			'name' => $this->defaultTableName,
			'create' => $this->elements
		];
	}

	public function setBackupData(array $data): bool{
		$this->defaultTableName = (string) $data['name'];
		$this->elements = (array) $data['create'];
		if($this->db->isTabExist($this->table)){
			we_backup_settings::i()->tablekeys[$this->table] = $this->db->getTableKeyArray($this->table);
			$this->db->delTable($this->table);
		}
		foreach($this->elements as &$cur){
			'@phan-var string $cur';
			if(substr($cur, 0, 6) === 'CREATE'){
				//Regex because of backups <6.2.4
				$cur = preg_replace('|(CREATE *\w* *`?)\w*' . stripTblPrefix($this->table) . '|i', '${1}' . $this->table, $cur, 1);
			}
		}

		//FIXME: this is NOT Save for MySQL Updates!!!!
		array_pop($this->elements); //get rid of old Engine statement
		$this->elements[] = ' ) ' . we_database_base::getCharsetCollation() . ' ENGINE=' . $this->db->getValidDBEngine();
		return ($this->db->query(implode(' ', $this->elements)));
	}

	protected function getTable(): string{
		return $this->defaultTableName;
	}

}
