<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 32f2411b88dc $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-09-03 13:00 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_base_delete{
	/** @var string[] */
	public static $foldersNotDel = [];

	/** @var int[] */
	public static $deletedItems = [];

	public static function checkDeleteEntry(int $id, string $table): bool{
		if($table === FILE_TABLE || $table === OBJECT_FILES_TABLE){
			return true;
		}
		return (fB('SELECT IsFolder FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id) ?
			self::checkDeleteFolder($id, $table) :
			self::checkDeleteFile($id, $table));
	}

	private static function checkDeleteFolder(int $id, string $table): bool{
		switch($table){
			case FILE_TABLE:
			case OBJECT_FILES_TABLE:
				return true;
			default:
				$ids = DB_WE::$inst->getAllq('SELECT ID FROM ' . DB_WE::$inst->escape($table) . ' WHERE ParentID=' . $id, true);
				foreach($ids as $id){
					if(!self::checkDeleteEntry((int) $id, $table)){
						return false;
					}
				}
				return true;
		}
	}

	private static function checkDeleteFile(int $id, string $table): bool{
		switch($table){
			case FILE_TABLE:
			case OBJECT_FILES_TABLE :
				return true;
			case OBJECT_TABLE :
				return !(we_contents_object::isUsedByObjectFile($id));
			case TEMPLATES_TABLE:
				$arr = we_rebuild_base::getTemplAndDocIDsOfTemplate($id, false, false, true);
				return (empty($arr['documentIDs']));
		}
		return true;
	}

	private static function deleteFolder(int $id, string $table, string $path, bool $delR): bool{
		$isTemplateFolder = ($table === TEMPLATES_TABLE);

		$path = $path ?: fS('SELECT Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id);
		if(!$path){
			return false;
		}

		if($delR){ // recursive delete
			$toDeleteArray = DB_WE::$inst->getAllq('SELECT ID FROM ' . DB_WE::$inst->escape($table) . ' WHERE ParentID=' . $id, true);
			foreach($toDeleteArray as $toDelete){
				self::deleteEntry((int) $toDelete, $table, true, false);
			}
		}

		// do not delete class folder if class still exists!
		if($table === OBJECT_FILES_TABLE){
			if(fB('SELECT IsClassFolder FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id) && // it is a class folder
				fB('SELECT 1 FROM ' . OBJECT_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($path) . '"')){ // class still exists
				return false;
			}
		}
		// Fast Fix for deleting entries from tblLangLink: #5840
		if(DB_WE::$inst->query('DELETE FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id)){
			DB_WE::$inst->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . DB_WE::$inst->escape(stripTblPrefix($table)) . '" AND DID=' . $id);
		}

		self::deleteContentFromDB($id, $table);
		if(substr($path, 0, 3) === '/..'){
			return false;
		}
		$file = ((!$isTemplateFolder) ? WE_DOCUMENT_ROOT : TEMPLATES_PATH) . $path;
		if($table === TEMPLATES_TABLE || $table === FILE_TABLE){
			if(!we_base_file::deleteLocalFolder($file)){
				self::$foldersNotDel[] = $file;
			}
		}
		switch($table){
			case FILE_TABLE:
				$file = WE_DOCUMENT_ROOT . WE_SITE_DIR . substr($path, 1);
				return we_base_file::deleteLocalFolder($file, true);
			case OBJECT_TABLE :
				if(($ofID = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($path) . '"'))){
					return self::deleteEntry($ofID, OBJECT_FILES_TABLE, true, false);
				}
				return false;
		}
		return true;
	}

	private static function deleteFile(int $id, string $table, string $path, string $contentType): bool{
		$isTemplateFile = ($table === TEMPLATES_TABLE);

		$path = $path ?: fS('SELECT Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id);
		self::deleteContentFromDB($id, $table);

		$file = ((!$isTemplateFile) ? WE_DOCUMENT_ROOT : TEMPLATES_PATH) . $path;

		switch($table){
			case FILE_TABLE:
				we_base_file::deleteLocalFile(WE_DOCUMENT_ROOT . WE_SITE_DIR . trim($path, '/'));
			//no break
			case TEMPLATES_TABLE:
				we_base_file::deleteLocalFile(preg_replace('/\.tmpl$/i', '.php', $file));
				DB_WE::$inst->query('DELETE FROM ' . CAPTCHADEF_TABLE . ' WHERE ID=' . $id);
				break;
		}

		DB_WE::$inst->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE (ID=' . $id . ' AND DocumentTable="' . DB_WE::$inst->escape(stripTblPrefix($table)) . '") OR (remObj=' . $id . ' AND remTable="' . DB_WE::$inst->escape(stripTblPrefix($table)) . '")');

		switch($table){
			case FILE_TABLE:
				$tbl = DB_WE::$inst->escape(stripTblPrefix(FILE_TABLE));

				DB_WE::$inst->query('DELETE FROM ' . CONTENT_TABLE . ' WHERE DID=' . $id);
				DB_WE::$inst->query('UPDATE ' . CONTENT_TABLE . ' c SET c.BDID=0 WHERE c.DocumentTable="' . $tbl . '" AND c.Type IN ("href","img") AND c.BDID=' . $id);
				DB_WE::$inst->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . $id);
				//	Delete entries from schedule as well

				we_modules_info::isActive(we_modules_info::SCHEDULER) && DB_WE::$inst->query('DELETE FROM ' . SCHEDULE_TABLE . ' WHERE DID=' . $id . ' AND documentTable="' . $tbl . '"');
				we_modules_info::isActive(we_modules_info::CUSTOMER) && DB_WE::$inst->query('DELETE FROM ' . CUSTOMER_FILTER_TABLE . ' WHERE modelTable="' . $tbl . '" AND modelId=' . $id);
				DB_WE::$inst->query('UPDATE ' . NAVIGATION_TABLE . ' SET LinkID=0 WHERE Selection="' . we_navigation_navigation::SELECTION_STATIC . '" AND SelectionType="' . we_navigation_navigation::STYPE_DOCLINK . '" AND LinkID=' . $id);
				// Fast Fix for deleting entries from tblLangLink: #5840
				// Fast Fix for deleting entries from tblLangLink: #5840
				DB_WE::$inst->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $tbl . '" AND ' . $id . ' IN (DID,LDID)');
				break;
			case VFILE_TABLE :
				DB_WE::$inst->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE ID=' . $id . ' AND DocumentTable="' . DB_WE::$inst->escape(stripTblPrefix(VFILE_TABLE)) . '" AND type IN ("collection","archive")'); //FIXME: delete OR archive
				break;
			case OBJECT_FILES_TABLE :
				$tableID = fI('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE IsClassFolder=0 AND ID=' . $id);
				DB_WE::$inst->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID>0 AND ID=' . $id);
				if($tableID){
					DB_WE::$inst->query('DELETE FROM ' . OBJECT_X_TABLE . $tableID . ' WHERE OF_ID=' . $id);
					DB_WE::$inst->query('UPDATE ' . NAVIGATION_TABLE . ' SET LinkID=0 WHERE Selection="' . we_navigation_navigation::SELECTION_STATIC . '" AND SelectionType="' . we_navigation_navigation::STYPE_OBJLINK . '" AND LinkID=' . $id);
					//Bug 2892
					$foo = DB_WE::$inst->getAllq('SELECT ID FROM ' . OBJECT_TABLE, true);
					foreach($foo as $testclassID){
						if(DB_WE::$inst->isColExist(OBJECT_X_TABLE . $testclassID, we_contents_object::QUERY_PREFIX . $tableID)){
							DB_WE::$inst->query('UPDATE ' . OBJECT_X_TABLE . intval($testclassID) . ' SET ' . we_contents_object::QUERY_PREFIX . $tableID . '=0 WHERE ' . we_contents_object::QUERY_PREFIX . $tableID . '=' . $id);
						}
					}
					// Fast Fix for deleting entries from tblLangLink: #5840
					DB_WE::$inst->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '" AND ' . $id . ' IN (DID,LDID)');
				}
				//	Delete entries from schedule as well
				if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
					DB_WE::$inst->query('DELETE FROM ' . SCHEDULE_TABLE . ' WHERE DID=' . $id . ' AND documentTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '"');
					DB_WE::$inst->query('DELETE FROM ' . CUSTOMER_FILTER_TABLE . ' WHERE modelTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '" AND modelId=' . $id);
				}

				break;
		}

		DB_WE::$inst->query('DELETE FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id);
		if($table === OBJECT_TABLE){
			$ofID = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($path) . '"');
			if($ofID){
				self::deleteEntry($ofID, OBJECT_FILES_TABLE, true, false);
			}
			DB_WE::$inst->delTable(OBJECT_X_TABLE . $id);
		}
		if($contentType === we_base_ContentTypes::IMAGE){
			we_contents_thumbnail::deleteByImageID($id);
		}
		return true;
	}

	public static function deleteEntry(int $id, string $table, bool $delR = true, bool $skipHook = false): bool{
		switch($table){
			case FILE_TABLE :
			case TEMPLATES_TABLE :
			case OBJECT_TABLE :
			case OBJECT_FILES_TABLE :
			case VFILE_TABLE :
				break;
			default:
				t_e('unable to delete files from this table', $table);
				exit('unsupported delete');
		}

		if(we_modules_info::isActive(we_modules_info::WORKFLOW) && ($table === FILE_TABLE || $table === OBJECT_FILES_TABLE)){
			if(we_workflow_utility::inWorkflow($id, $table)){
				we_workflow_utility::removeDocFromWorkflow($id, $table, we_users_user::getID(), g_l('modules_workflow', '[doc_deleted]'));
			}
		}

		if(!$id){
			return false;
		}
		$row = getHash('SELECT Path,IsFolder,ContentType FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $id);
		if(!$row){
			self::$deletedItems[] = $id;
			return false;
		}
		$isVersioned = we_versions_version::CheckPreferencesCtypes($row['ContentType']);
		//no need to init doc, if no version is needed or hook is executed
		$object = ($isVersioned || !$skipHook ? we_exim_contentProvider::getInstance($row['ContentType'], $id, $table) : null);

		if($isVersioned && $object){
			if(!we_versions_version::versionExists($id, $table)){
				assert($object instanceof we_contents_base);
				we_versions_version::saveVersion($object);
			}

			we_versions_version::setVersionOnDelete($id, $table, $row['ContentType']);
		}
		/* hook */
		if(!$skipHook){
			$hook = new we_hook_handler('delete', [$object]);
			$hook->executeHook();
		}

		self::deleteTemporaryContent($id, $table);

		if($row['IsFolder']){
			self::deleteFolder($id, $table, $row['Path'], $delR);
		} else {
			self::deleteFile($id, $table, $row['Path'], $row['ContentType']);
		}
		self::$deletedItems[] = $id;
		return true;
	}

	/**
	 * @param int $id
	 * @param string $table
	 * @return bool true on success, or if not in DB
	 */
	public static function deleteContentFromDB(int $id, string $table): bool{
		if(!fB('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE DID=' . $id . ' AND DocumentTable="' . DB_WE::$inst->escape(stripTblPrefix($table)) . '" LIMIT 1')){
			return true;
		}

		return DB_WE::$inst->query('DELETE FROM ' . CONTENT_TABLE . ' WHERE DID=' . $id . ' AND DocumentTable="' . DB_WE::$inst->escape(stripTblPrefix($table)) . '"');
	}

	/**
	 *
	 * @param array{IsFolder:int,Path:string,hasFiles:bool} $idInfos
	 * @param string $table
	 * @return bool
	 */
	private static function getHasPerm(array $idInfos, string $table): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		switch($table){
			case FILE_TABLE:
				return (
					($idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_FOLDER_DELETE]) && !$idInfos['hasFiles']) ||
					(!$idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_DELETE])) ||
					($idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_FOLDER_DELETE, we_base_perm::WORK_DOCUMENT_DELETE], false) && $idInfos['hasFiles'])
					);
			case TEMPLATES_TABLE:
				return (
					($idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::WORK_TEMP_FOLDER_DELETE]) && !$idInfos['hasFiles']) ||
					(!$idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATE_DELETE])) ||
					($idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::WORK_TEMP_FOLDER_DELETE, we_base_perm::WORK_TEMPLATE_DELETE], false) && $idInfos['hasFiles'])
					);
			case OBJECT_FILES_TABLE:
				return (we_base_permission::hasPerm([we_base_perm::OBJECTFILE_DELETE]));
			case OBJECT_TABLE:
				return ($idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::OBJECT_DELETE]));
			case VFILE_TABLE:
				return (
					($idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_FOLDER_DELETE]) && !$idInfos['hasFiles']) ||
					(!$idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_DELETE])) ||
					($idInfos['IsFolder'] && we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_FOLDER_DELETE, we_base_perm::WORK_COLLECTION_DELETE], false) && $idInfos['hasFiles'])
					);
			default:
				return false;
		}
	}

	/**
	 *
	 * @param int[] $selectedItems
	 * @param string $table
	 * @param array<int,string> $users
	 * @param array<int,string> $objects
	 * @param int $selectedItem
	 * @return int
	 */
	private static function checkFilePerm(array $selectedItems, string $table, array &$users, array &$objects, int &$selectedItem): int{
		foreach($selectedItems as $selectedItem){
			if(!we_base_permission::checkIfRestrictUserIsAllowed($selectedItem, $table)){
				return we_base_permission::USER_RESTRICTED;
			}

			if(!we_base_delete::checkDeleteEntry($selectedItem, $table)){
				return we_base_permission::USED_BY_OTHER;
			}
		}

		foreach($selectedItems as $selectedItem){
			switch($table){
				case FILE_TABLE:
					if(($users = we_users_util::getUsersForDocWorkspace([$selectedItem]))){
						return we_base_permission::WORKSPACE_HAS_USERS;
					}

					// check if childrenfolders are workspaces
					$childs = [];

					we_base_util::pushChilds($childs, $selectedItem, $table, true);

					if(($users = we_users_util::getUsersForDocWorkspace($childs))){
						return we_base_permission::SUBWORKSPACE_HAS_USERS;
					}

					if(($objects = self::getObjectsForDocWorkspace([$selectedItem]))){
						return we_base_permission::WORKSPACE_FOR_OBJECT;
					}

					$childs = [];

					we_base_util::pushChilds($childs, $selectedItem, $table, true);

					if(($objects = self::getObjectsForDocWorkspace($childs))){
						return we_base_permission::WORKSPACE_USED_BY_OBJECTS;
					}

					break;

				case TEMPLATES_TABLE:
					if(($users = we_users_util::getUsersForDocWorkspace([$selectedItem], 'workSpaceTmp'))){
						return we_base_permission::WORKSPACE_HAS_USERS;
					}

					// check if childrenfolders are workspaces
					$childs = [];

					we_base_util::pushChilds($childs, $selectedItem, $table, true);
					$users = we_users_util::getUsersForDocWorkspace($childs, 'workSpaceTmp');

					if(($users = array_unique($users))){
						return we_base_permission::SUBWORKSPACE_HAS_USERS;
					}
					break;

				case OBJECT_FILES_TABLE:
					if(($users = we_users_util::getUsersForDocWorkspace([$selectedItem], 'workSpaceObj'))){
						return we_base_permission::WORKSPACE_HAS_USERS;
					}

					$childs = [];
					we_base_util::pushChilds($childs, $selectedItem, $table, true);

					if(($users = we_users_util::getUsersForDocWorkspace($childs, 'workSpaceObj'))){
						return we_base_permission::SUBWORKSPACE_HAS_USERS;
					}
					break;
			}
		}
		return 0;
	}

	/**
	 *
	 * @param int[] $ids
	 * @return array<int,string>
	 */
	private static function getObjectsForDocWorkspace(array $ids): array{
		if(!we_modules_info::isActive(we_modules_info::OBJECT)){
			return [];
		}

		return DB_WE::$inst->getAllFirstq('SELECT of.ID,of.Path FROM ' . OBJECT_FILES_TABLE . ' `of` JOIN ' . FILE_TABLE . ' f ON FIND_IN_SET(f.ID,of.Workspaces) WHERE f.ID IN(' . implode(',', $ids) . ')', false); //@phan-suppress-current-line PhanPartialTypeMismatchReturn
	}

	/**
	 *
	 * @param we_base_jsCmd $jsCmd
	 * @param int $retVal
	 * @param string $elemPath
	 * @param string $table
	 * @param array<int,string> $objects
	 * @param array<int,string> $users
	 */
	private static function handleDeleteNotAllowed(we_base_jsCmd $jsCmd, int $retVal, string $elemPath, string $table, array $objects, array $users): void{
		switch($retVal){
			case we_base_permission::NO_PERMISSION:
				$jsCmd->addMsg(g_l('alert', '[no_perms_action]'), we_base_util::WE_MESSAGE_ERROR);
				return;
			case we_base_permission::WORKSPACE_USED_BY_OBJECTS: //	not allowed to delete workspace
				$objList = '';
				foreach($objects as $val){
					$objList .= '- ' . $val . "\n";
				}
				$jsCmd->addMsg(sprintf(g_l('alert', '[delete][workspace][object_r]'), $elemPath, $objList), we_base_util::WE_MESSAGE_ERROR);
				return;
			case we_base_permission::SUBWORKSPACE_HAS_USERS: //	not allowed to delete workspace
				$usrList = '';
				foreach($users as $val){
					$usrList .= '- ' . $val . "\n";
				}
				$jsCmd->addMsg(sprintf(g_l('alert', '[delete][workspace][user_r]'), $elemPath, $usrList), we_base_util::WE_MESSAGE_ERROR);
				return;
			case we_base_permission::WORKSPACE_FOR_OBJECT: //	not allowed to delete workspace
				$objList = '';
				foreach($objects as $val){
					$objList .= '- ' . $val . "\n";
				}
				$jsCmd->addMsg(sprintf(g_l('alert', '[delete][workspace][object]'), $elemPath, $objList), we_base_util::WE_MESSAGE_ERROR);
				return;
			case we_base_permission::WORKSPACE_HAS_USERS: //	not allowed to delete workspace
				$usrList = '';
				foreach($users as $val){
					$usrList .= '- ' . $val . "\n";
				}
				$jsCmd->addMsg(sprintf(g_l('alert', '[delete][workspace][user]'), $elemPath, $usrList), we_base_util::WE_MESSAGE_ERROR);
				return;
			case we_base_permission::USER_RESTRICTED: //	not allowed to delete document
				$jsCmd->addMsg(sprintf(g_l('alert', '[noRightsToDelete]'), $elemPath), we_base_util::WE_MESSAGE_ERROR);
				return;
			case we_base_permission::USED_BY_OTHER:
			default:

				//unhandled error
				switch($table){
					case TEMPLATES_TABLE:
						$jsCmd->addMsg(g_l('alert', '[delete][notok][Templ_used]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					case OBJECT_TABLE:
						$jsCmd->addMsg(g_l('alert', '[delete][notok][Class_used]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					default:
						$jsCmd->addMsg(g_l('alert', '[delete][notok][default]'), we_base_util::WE_MESSAGE_ERROR);
				}
				return;
		}
	}

	//FIXME: this is the only version that checks for permissions

	/**
	 *
	 * @param we_base_jsCmd $jsCmd
	 * @param int[] $selectedItems
	 * @param string $table
	 * @param string $wecmd0
	 * @return int
	 */
	public static function doDeleteSelected(we_base_jsCmd $jsCmd, array $selectedItems, string $table, string $wecmd0): int{
		//	look which documents must be deleted.
		$idInfos = [
			'IsFolder' => 0,
			'Path' => '',
			'hasFiles' => false
		];
		if($selectedItems && in_array($table, [FILE_TABLE, TEMPLATES_TABLE, VFILE_TABLE], true)){
			$idInfos = getHash('SELECT IsFolder,Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . intval($selectedItems[0]));
			if(!$idInfos){
				t_e('ID ' . $selectedItems[0] . ' not present in table ' . $table);
				return 0;
			}
			if($idInfos['IsFolder']){
				$idInfos['hasFiles'] = fB('SELECT 1 FROM ' . DB_WE::$inst->escape($table) . ' WHERE ParentID=' . intval($selectedItems[0]) . ' AND IsFolder=0 AND Path LIKE "' . DB_WE::$inst->escape($idInfos['Path']) . '%" LIMIT 1');
			}
		}
		$hasPerm = self::getHasPerm($idInfos, $table);
		unset($idInfos);
		$users = $objects = [];
		$selectedItem = 0;
		$retVal = !$hasPerm ? we_base_permission::NO_PERMISSION : self::checkFilePerm($selectedItems, $table, $users, $objects, $selectedItem);

		if($retVal !== 0){
			$elemPath = id2path($selectedItem, $table);
			self::handleDeleteNotAllowed($jsCmd, $retVal, $elemPath, $table, $objects, $users);
			return $retVal;
		}

		self::$foldersNotDel = self::$deletedItems = [];

		foreach($selectedItems as $sel){
			we_base_delete::deleteEntry($sel, $table, true, false);
		}

		if(self::$deletedItems){
			$deleted_objects = [];

			if($table === OBJECT_TABLE){ // close all open objects, if a class is deleted
				$localDeletedItems = [];

				// if its deleted and not selected, it must be an object
				foreach(self::$deletedItems as $cur){
					if(in_array($cur, $selectedItems)){
						$localDeletedItems[] = $cur;
					} else {
						$deleted_objects[] = $cur; // deleted objects when classes are deleted
					}
				}
				self::$deletedItems = $localDeletedItems;

				//some extra work on object_files if a class is deleted
				if(!empty($deleted_objects)){
					we_customer_documentFilter::deleteModel($deleted_objects, OBJECT_FILES_TABLE);
					we_base_history::deleteFromHistory($deleted_objects, OBJECT_FILES_TABLE);
				}
				$jsCmd->addCmd('closeDeletedEntries', OBJECT_FILES_TABLE, $deleted_objects);
			}

			// delete the customerfilters
			we_modules_info::isActive(we_modules_info::CUSTOMER) && we_customer_documentFilter::deleteModel(self::$deletedItems, $table);
			we_base_history::deleteFromHistory(self::$deletedItems, $table);

			$jsCmd->addCmd('closeDeletedEntries', $table, self::$deletedItems);
			if($wecmd0 === 'do_delete'){ // delete from select tree
				$jsCmd->addCmd('del_continue', $table);
			} else {
				$jsCmd->addCmd('load', $table);
			}
		}

		if(we_editor_functions::isNormalMode()){ //	different messages in normal or seeMode
			if(self::$foldersNotDel){
				$_SESSION['weS']['delete_files'] = [
					'nok' => [],
					'info' => str_replace(["\n", '\n'], '', sprintf(g_l('alert', '[folder_not_empty]'), ''))
				];
				foreach(self::$foldersNotDel as $datafile){
					$_SESSION['weS']['delete_files']['nok'][] = [
						'ContentType' => we_base_ContentTypes::FOLDER_DOCUMENT,
						'path' => $datafile
					];
				}
				$jsCmd->addCmd('delInfo');
			} else {
				$jsCmd->addMsg(g_l('alert', '[delete][ok]'), we_base_util::WE_MESSAGE_NOTICE);
			}
		}
		return 0;
	}

	public static function getDialog(): void{
		$table = we_base_request::_(we_base_request::TABLE, 'we_cmd', '', 2);
		$wfchk = we_modules_info::isActive(we_modules_info::WORKFLOW) && ($table === FILE_TABLE || $table === OBJECT_FILES_TABLE) ?
			we_base_request::bool('we_cmd', false, 3) :
			1;
		$wecmd0 = we_base_request::string('we_cmd', '', 0);
		$jsCmd = new we_base_jsCmd();
		$selectedItems = we_base_request::intList('sel', []);
		$retVal = 0;
		$found = false;

		if(!$wfchk){
			if($selectedItems){
				foreach($selectedItems as $selectedItem){
					'@phan-var int $selectedItem';
					if(we_workflow_utility::inWorkflow($selectedItem, $table)){
						$found = true;
						break;
					}
				}
			}
		} elseif(in_array($wecmd0, ['do_delete', 'delete_single_document'], true)){
			if($selectedItems){
				$retVal = self::doDeleteSelected($jsCmd, $selectedItems, $table, $wecmd0);
			} else {
				$jsCmd->addMsg(g_l('alert', '[nothing_to_delete]'), we_base_util::WE_MESSAGE_WARNING);
			}
		}

//	in seeMode return to startDocument ...

		if(we_editor_functions::isSeeMode()){
			if($retVal !== 0){
				$jsCmd->addMsg(g_l('alert', '[delete][single][return_to_start]'), we_base_util::WE_MESSAGE_NOTICE);
				//	document deleted -> go to seeMode startPage
				$jsCmd->addCmd('we_cmd', ['start_multi_editor']);
			} else {
				$jsCmd->addMsg(g_l('alert', '[delete][single][no_delete]'), we_base_util::WE_MESSAGE_ERROR);
			}
			$body = we_html_element::body();
		} elseif(!$wfchk && $wecmd0 != 'delete'){
			$body = we_html_element::body(['onload' => ($selectedItems ? 'confirmDel(' . ($found ? 1 : 0) . ')' : '')], '<form name="' . we_base_constants::WE_FORM . '" method="post">' . we_html_element::hidden('sel', implode(',', $selectedItems)) . '</form>');
		} elseif($wecmd0 === 'do_delete'){
			$body = we_html_element::body();
		} else {
			$body = we_html_element::body(['class' => 'weTreeHeader'], we_html_element::div([], we_html_element::h(1, htmlspecialchars(g_l('weClass', '[deleteMode][title]')), [
							'class' => 'big', 'style' => "padding:0;margin:0;"]) .
						we_html_element::p(['class' => 'small'], we_html_element::span(['class' => 'middlefont'], g_l('weClass', '[deleteMode][text]'))) .
						we_html_element::div([], we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:WE().util.showConfirm(window, '', '" . g_l('alert', '[delete][selected]') . "',['do_delete','','" . $table . "']);"), '', we_html_button::create_button('quit_delete', "javascript:we_cmd('exit_delete','','" . $table . "')")))) .
					we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post'], we_html_element::hidden('sel', '')));
		}

		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'delete.js', 'init();', ['id' => 'loadVarDelete', 'data-deleteData' => setDynamicVar([
					'table' => $table,
					'wecmd0' => $wecmd0
			])]) .
			we_html_element::jsScript(JS_DIR . 'pushTopCmd.js') .
			$jsCmd->getCmds(), $body);
	}

	public static function getDelInfo(): void{
		if(empty($_SESSION['weS']['delete_files'])){
			//this should not happen
			return;
		}
		$table = new we_html_table(['style' => 'margin:10px;', 'class' => 'default withSpace'], 1, 2);
		foreach($_SESSION['weS']['delete_files']['nok'] as $i => $data){
			$table->setColAttributes($i, 0, ['class' => 'selectoricon', 'data-contenttype' => (empty($data['ContentType']) ? '' : $data['ContentType'] )])
				->setCol($i, 1, str_replace(WE_DOCUMENT_ROOT, '', $data['path']));
		}

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox((string) $_SESSION['weS']['delete_files']['info'], we_html_tools::TYPE_ALERT, 500))))
			->add((new we_html_multiIconBoxContentLine('', we_html_element::div(['class' => 'blockWrapper', 'style' => 'width: 475px; height: 350px; border:1px #dce6f2 solid;'], $table->getHtml()))));
		unset($_SESSION['weS']['delete_files']);

		$buttons = new we_html_table(['class' => 'default', 'style' => "text-align:right"], 1, 1, [new we_html_tr([new we_html_td(we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();"))])]
		);
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('setIconOfDocClass', 'selectoricon');
		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'dialogPushCmd.js'), we_html_element::body(['class' => 'weDialogBody'], (new we_html_multiIconBox($parts))->marginLeft()->buttons($buttons->getHtml())->get() . $jsCmd->getCmds()
			)
		);
	}

	public static function deleteTemporaryContent(int $documentID, string $table, int $tableID = 0): bool{
		switch($table){
			case FILE_TABLE:
				return DB_WE::$inst->query('DELETE FROM ' . CONTENT_TABLE . ' WHERE DocumentTable="' . TEMPORARY_DOCTARGET . '" AND DID=' . $documentID);
			case OBJECT_FILES_TABLE:
				$tableID = $tableID ?: fI('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE IsClassFolder=0 AND ID=' . $documentID);
				return DB_WE::$inst->query('DELETE FROM ' . OBJECT_X_TABLE . $tableID . ' WHERE OF_DocumentTable="' . TEMPORARY_DOCTARGET . '" AND OF_ID=' . $documentID);
		}
		return false;
	}

}
