<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 1ea9bd0384a8 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-08-17 12:38 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_doclist_searchObjectfiles extends we_search_search{
	/** @var int */
	var $height = 0;

	/** @var string[] */
	var $searchname = [];

	/** @var string[] */
	var $searchlocation = [];

	/** @var string[] */
	var $searchfield = [];

	/** @var string */
	var $show;

	/** @var bool */
	protected $greenOnly = false;

	/** @var int */
	protected $workspaceID = 0;

	/** @var int */
	protected $tableID = 0;

	/** @var string[] */
	private static $intFields = [];

	/** @var string[] */
	private static $realFields = [];

	/** @var array<int,string|array<string,mixed>> */
	public $objsearch = [];

	/** @var string[] */
	public $objsearchField = [];

	/** @var array<int,string> */
	public $objlocation = [];

	function init(array $sessDat = []): void{
		foreach($sessDat as $v){
			$this->{$v} = $v;
		}
	}

	function getFields(string $whichSearch = '', int $tableID = 0): array{
		if(!$tableID){
			return [];
		}

		$props = [];
		$fields = [];
		$tableInfoFields = DB_WE::$inst->metaFieldNames(OBJECT_X_TABLE . $tableID);
		'@phan-var string[] $tableInfoFields';
		$tableInfoProperties = DB_WE::$inst->metaFieldNames(OBJECT_FILES_TABLE);
		'@phan-var string[] $tableInfoProperties';

		foreach($tableInfoProperties as $cur){
			switch($cur){
				case 'Filename':
					$props['`of`.' . $cur] = g_l('modules_object', '[objectname]');
					break;
				case 'Path':
					$props['`of`.' . $cur] = g_l('modules_object', '[objectpath]');
					break;
				case 'ID':
					$props['`of`.' . $cur] = g_l('modules_object', '[objectid]');
					break;
				case 'Url':
					$props['`of`.' . $cur] = g_l('modules_object', '[objecturl]');
					break;
			}
		}
		$all = array_keys($props);

		foreach($tableInfoFields as $cur){
			$regs = explode('_', $cur, 2);
			switch($regs[0]){
				case 'OF':
				case 'variant':
				case 'object':
				case we_contents_objectBase::TYPE_MULTIOBJECT:
					continue 2;
				default:
					if(count($regs) === 2){
						$fields['obx.' . $cur] = $regs[1];
					}
					$all[] = 'obx.' . $cur;
			}
		}

		return $props + [implode(',', $all) => g_l('modules_object', '[allFields]')] + $fields;
	}

	function getGreenOnly(): string{
		if($this->greenOnly){
			return we_contents_objectFile::makePIDTail($this->workspaceID, $this->tableID) . ' AND of.IsPublished=1 AND ';
		}

		return '';
	}

	/**
	 *
	 * @param array<int,string|array> $searchnames
	 * @param string[] $searchfield
	 * @param string[] $searchlocation
	 * @param string $tablename
	 * @param int $rows
	 * @param int $start
	 * @param string $order
	 * @param bool $desc
	 * @param bool $fieldsQualified
	 * @return string
	 */
	function searchfor(array $searchnames, array $searchfield, array $searchlocation, string $tablename, string $alias = '', int $rows = -1, int $start = 0, string $order = '', bool $desc = false, bool $fieldsQualified = true): string{
		$realFields = $this->getRealFields($tablename);
		$intFields = $this->getIntFields($tablename);
		$allFields = $this->db->metaFieldNames($tablename);

		foreach($searchnames as $i => $searchname){
			$filteredFields = [];
			$fieldsToFilterOut = [];

			$type = is_array($searchname) ? 0 : (!preg_match('/^[-+]?\d*\.?\d+$/', $searchname) ? 1 : (!preg_match('/^[-+]?\d+$/', $searchname) ? 2 : 0));
			switch($type){
				case 1:
					$fieldsToFilterOut = array_merge($fieldsToFilterOut, $realFields);
				//no break!
				case 2:
					$fieldsToFilterOut = array_merge($fieldsToFilterOut, $intFields);
			}

			$arrSearchfield = explode(',', trim($searchfield[$i], ','));
			foreach($arrSearchfield as $f){
				if(!in_array($f, $fieldsToFilterOut)){
					$filteredFields[] = (in_array($f, $allFields) ? $alias . '.' : '') . $f;
				}
			}
			$searchfield[$i] = implode(',', $filteredFields);
		}

		return parent::searchfor($searchnames, $searchfield, $searchlocation, '', $alias, $rows, $start, '', false, true);
	}

	private function getIntFields(string $tablename): array{
		if(self::$intFields || !$tablename){
			return self::$intFields;
		}

		foreach($this->db->metadata($tablename) as $f){
			if(in_array($f['type'], [we_database_base::TYPE_INT, we_database_base::TYPE_TINYINT, we_database_base::TYPE_SMALLINT, we_database_base::TYPE_MEDIUMINT, we_database_base::TYPE_BIGINT], true)){
				self::$intFields[] = $f['name'];
			}
		}
		return self::$intFields;
	}

	private function getRealFields(string $tablename): array{
		if(self::$realFields || !$tablename){
			return self::$realFields;
		}

		foreach($this->db->metadata($tablename) as $f){
			if(in_array($f['type'], [we_database_base::TYPE_DOUBLE, we_database_base::TYPE_FLOAT, we_database_base::TYPE_DECIMAL], true)){
				self::$realFields[] = $f['name'];
			}
		}
		return self::$realFields;
	}

	function removeFilter(int $position): void{
		if($this->height == 0){
			$this->objsearch = [];
			$this->objsearchField = [];
			$this->objlocation = [];
			return;
		}

		foreach(array_keys($this->objsearch) as $idx){
			if($idx == $position){
				unset($this->objsearch[$position]);
			} elseif($idx >= $position){
				$this->objsearch[($idx - 1)] = $this->objsearch[$idx];
				unset($this->objsearch[$idx]);
			}
		}

		foreach(array_keys($this->objsearchField) as $idx){
			if($idx == $position){
				unset($this->objsearchField[$position]);
			} elseif($idx >= $position){
				$this->objsearchField[($idx - 1)] = $this->objsearchField[$idx];
				unset($this->objsearchField[$idx]);
			}
		}

		foreach(array_keys($this->objlocation) as $idx){
			if($idx == $position){
				unset($this->objlocation[$position]);
			} elseif($idx >= $position){
				$this->objlocation[($idx - 1)] = $this->objlocation[$idx];
				unset($this->objlocation[$idx]);
			}
		}

		$this->height--;
	}

	/**
	 *
	 * @param we_doclist_model $model
	 * @param string $whichSearch
	 * @return array<int,string[]>
	 */
	public function searchFields(we_doclist_model $model, string $whichSearch): array{
		$folderProps = $this->db->getHash('SELECT Path,TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $model->getPropertyInt('currentFolderID'));
		$this->Order = (string) $model->getProperty('currentOrder');
		$this->searchname = array_map('trim', $model->getProperty('currentSearch'));
		$this->searchfield = $model->getProperty('currentSearchFields');
		$this->searchlocation = $model->getProperty('currentLocation');
		$this->searchstart = $model->getPropertyInt('currentSearchstart');
		$this->anzahl = $model->getPropertyInt('currentAnzahl');
		$this->workspaceID = $model->getPropertyInt('currentWorkspace');
		$this->greenOnly = $model->getPropertyB('currentGreenOnly');
		$this->tableID = $folderProps['TableID'];

		$where = (!empty($this->searchname) ? $this->searchfor($this->searchname, $this->searchfield, $this->searchlocation, OBJECT_X_TABLE . $this->tableID, 'obx', $this->anzahl, $this->start) . ' AND ' : '');
		$whereRestrictOwners = ' AND (of.RestrictOwners=0 OR of.CreatorID=' . we_users_user::getID() . ' OR FIND_IN_SET(' . we_users_user::getID() . ',of.Owners)) ';

		$this->table = (OBJECT_X_TABLE . $this->tableID . ' obx JOIN ' . OBJECT_FILES_TABLE . ' `of` ON (obx.OF_ID=of.ID AND obx.OF_DocumentTable="tblObjectFiles")');
		$this->where = $where . $this->getGreenOnly() . 'of.Path LIKE "' . $folderProps['Path'] . '/%" AND of.ID!=0 AND of.IsFolder=0 ' . $whereRestrictOwners;
		$this->get = 'obx.*,of.ID,of.Filename,of.Path,of.ParentID,of.TableID,of.Workspaces,of.Publish_Date,of.IsSearchable,of.Language,of.Url,of.TriggerID,of.Mod_Date,of.WebUserID';

		return $this->searchquery();
	}

	/**
	 * @param we_search_modelBase $model
	 * @param string $whichSearch
	 * @return we_search_resultBase[]
	 */
	function searchProperties(we_search_modelBase $model, string $whichSearch): array{
		assert($model instanceof we_doclist_model);

		$resultObjects = [];
		$DefaultValues = we_unserialize(fS('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->tableID));
		$workspaceFlag = empty($DefaultValues['WorkspaceFlag']) ? '' : $DefaultValues['WorkspaceFlag'];

		foreach($this->searchFields($model, $whichSearch) as $result){
			$resultObject = new we_doclist_resultObjectfiles($this);
			switch($whichSearch){
				case we_doclist_view::DOCLIST_OBJECTFILES:
					break;
				// for future ObjectSearch
				/*
				  case we_search_view::SEARCH_OBJECTFILES:
				  $resultObject = new we_search_resultObjectfiles($this);
				  break;
				 *
				 */
				default:
					continue 2;
			}

			$show = ($result['Workspaces'] && we_users_util::in_workspace($this->workspaceID, we_base_request::filterIntList($result['Workspaces']), FILE_TABLE)) ||
				(!$result['Workspaces'] && $workspaceFlag);

			$resultObject->setID((int) $result['ID'])
				->setTableID((int) $result['TableID'])
				->setPath((string) $result['Path'])
				->setText((string) $result['Filename'])
				->setShow($show)
				->setIsSearchable((bool) $result['IsSearchable'])
				->setPublishDate((string) $result['Publish_Date'])
				->setModDate((string) $result['Mod_Date'])
				->setWorkspaces((string) $result['Workspaces'])
				->setUrl((string) $result['Url'])
				->setTriggerID((int) $result['TriggerID'])
				->setLanguage((string) $result['Language'])
				->setWebUserID(((int) $result['WebUserID']));

			foreach(we_doclist_resultObjectfiles::getFieldColumns((int) $result['TableID']) as $field){
				switch($field['type'] ?? ''){
					case 'int':
					case 'img':
					case 'audio':
					case 'video':
					case 'binary':
					case 'collection':
					case 'object':
						$resultObject->setFieldI($field['name'], (int) ($result[$field['name']] ?? 0));
						break;
					case 'input':
					case 'text':
					case 'href':
					case 'link':
					case 'country':
					case 'language':
					case 'multiobject': // INTLIST
					case 'select':
					case 'shopCategory':
						$resultObject->setFieldS($field['name'], (string) $result[$field['name']]);
						break;
					case 'date':
						$resultObject->setFieldD($field['name'], (new DateTime())->setTimestamp((int) $result[$field['name']]));
						break;
					case 'checkbox':
						$resultObject->setFieldB($field['name'], (bool) $result[$field['name']]);
						break;
					case 'float':
					case 'shopVat':
						$resultObject->setFieldF($field['name'], (float) $result[$field['name']]);
						break;
				}
			}

			$resultObject->processHtml();
			$resultObjects[] = $resultObject;
		}

		return $resultObjects;
	}

	/**
	 * @return array<int,mixed>
	 */
	private function searchquery(): array{
		if(!$this->table){
			throw new we_exception_database('no table set');
		}
		$this->maxItems = fI('SELECT COUNT(1) FROM ' . $this->table . ' ' . ($this->where ? ' WHERE ' . $this->where : ''));
		$this->founditems = $this->maxItems;

		//FIXME: this is a bad fix
		$count = 0;
		$ord = str_replace(' DESC', '', $this->Order, $count);

		return $this->db->getAllq('SELECT ' .
				$this->get . '
FROM ' . $this->table . ' ' . ($this->where ? ' WHERE ' . $this->where : '') . ' ' . ($this->Order ? ' ORDER BY `' . $ord . '`' . ($count ? ' DESC' : '') : '') . '
LIMIT ' . $this->searchstart . ',' . $this->anzahl);
	}

}
