<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 06f4047033e8 $
 * $Author: Lukas Imhof $
 * $Date: 2020-06-03 18:46 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_createObjectTemplate extends we_editor_base{
	/** @var int[] */
	private $usedIDs = [];

	/** @var string */
	protected $filename = '';

	/** @var int */
	protected $pid = 0;

	/** @var we_contents_root */
	private $doc;

	public static function cmd(): void{
		we_base_WE::$transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', 0, 3);
		$editor = new self(new we_contents_template());
		$jscmd = $editor->getJsCmd();

		$nr = we_base_request::int('we_cmd', 0, 2);
		$sid = we_base_request::string('SID');

		$editor->we_doc = new we_contents_template();
		$editor->we_doc->Table = TEMPLATES_TABLE;
		$editor->we_doc->we_new();

		$editor->filename = we_base_request::_(we_base_request::FILE, 'we_' . $sid . '_Filename');

		$editor->we_doc->Filename = $editor->filename;
		$editor->we_doc->Extension = '.tmpl';
		$editor->we_doc->setParentID(we_base_request::int('we_' . $sid . '_ParentID'));
		$editor->we_doc->ContentType = we_base_ContentTypes::TEMPLATE;

		$editor->we_doc->Table = TEMPLATES_TABLE;

//  $_SESSION['weS']['content'] is only used for generating a default template, it is
//  set in WE_OBJECT_MODULE_PATH\we_object_createTemplate.inc.php
		$editor->we_doc->setElementS('data', $_SESSION['weS']['content']);
		unset($_SESSION['weS']['content']);


		if(($we_responseText = $editor->we_doc->checkFieldsOnSave())){
			$jscmd->addCmd($we_responseText, we_base_util::WE_MESSAGE_ERROR);
			echo $editor->show();
			return;
		}
		if($editor->we_doc->we_save()){
			$jscmd->addMsg(sprintf(g_l('weEditor', '[' . $editor->we_doc->ContentType . '][response_save_ok]'), $editor->we_doc->Path), we_base_util::WE_MESSAGE_NOTICE);
			$jscmd->addCmd('we_cmd', ['object_changeTempl_ob', $nr, $editor->we_doc->ID]);
			$jscmd->addCmd('close_dialog');
			echo $editor->show();
			return;
		}
		$we_responseText = sprintf(g_l('weEditor', '[' . $editor->we_doc->ContentType . '][response_save_notok]'), $editor->we_doc->Path);
		$jscmd->addCmd($we_responseText, we_base_util::WE_MESSAGE_ERROR);
		echo $editor->show();
	}

	private function getObjectTags(int $tableID, bool $isField = false): string{
		$tableInfo = we_contents_objectFile::getSortedTableInfo($tableID, true, DB_WE::$inst);
		$content = '<table style="border:1px solid black;width:400px">';
		$regs = [];
		foreach($tableInfo as $cur){
			if(preg_match('/(.+?)_(.*)/', $cur['name'], $regs)){
				$content .= $this->getTmplTableRow($regs[1], $regs[2], $isField);
			}
		}
		$content .= '</table>';
		return $content;
	}

	private function getMultiObjectTags(string $name): string{
		$id = $this->doc->getElementI(we_contents_objectBase::TYPE_MULTIOBJECT . '_' . $name . 'class');
		if(!$id){
			return '';
		}
		$tableInfo = we_contents_objectFile::getSortedTableInfo($id, true, DB_WE::$inst);
		$content = '<table style="border:1px solid black;width:400px">';

		$regs = [];
		foreach($tableInfo as $cur){
			if(preg_match('/(.+?)_(.*)/', $cur['name'], $regs)){
				$content .= $this->getTmplTableRow($regs[1], $regs[2], true);
			}
		}
		$content .= '</table>';
		return $content;
	}

	private function getTemplTag(string $type, string $name, bool $isField = false): string{
		switch($type){
			case we_contents_object::TYPE_SELECT:
				return $isField ? '<we:field type="select" name="' . $name . '"/>' : '<we:var type="select" name="' . $name . '"/>';
			default:
			case we_contents_object::TYPE_INPUT:
			case we_contents_object::TYPE_TEXT:
			case we_contents_object::TYPE_INT:
			case we_contents_object::TYPE_FLOAT:
				return $isField ? '<we:field name="' . $name . '"/>' : '<we:var name="' . $name . '"/>';
			case we_contents_object::TYPE_LINK:
				return $isField ? '<we:field type="link" name="' . $name . '"/>' : '<we:var type="link" name="' . $name . '"/>';
			case we_contents_object::TYPE_HREF:
				return $isField ? '<we:field type="href" name="' . $name . '"/>' : '<we:var type="href" name="' . $name . '"/>';
			case we_contents_object::TYPE_IMG:
				return $isField ? '<we:field type="img" name="' . $name . '"/>' : '<we:var type="img" name="' . $name . '"/>';
			case we_contents_object::TYPE_CHECKBOX:
				return $isField ? '<we:field type="checkbox" name="' . $name . '"/>' : '<we:var type="checkbox" name="' . $name . '"/>';
			case we_contents_object::TYPE_DATE:
				return $isField ? '<we:field type="date" name="' . $name . '"/>' : '<we:var type="date" name="' . $name . '"/>';
			case we_contents_object::TYPE_OBJECT:
				return (!in_array($name, $this->usedIDs) ?
					$this->getObjectTags(intval($name), $isField) : '');
			case we_contents_objectBase::TYPE_MULTIOBJECT:
				return $this->getMultiObjectTags($name);
		}
	}

	private function getTmplTableRow(string $type, string $name, bool $isField = false): string{
		if($type != we_contents_objectBase::TYPE_MULTIOBJECT){
			return ''; /*
			  <tr>
			  <td style="width:100px;"><b>' . (($type != "object") ? $name : "") . '</b></td>
			  <td style="width:300px;">' . $this->getTemplTag($type, $name, $isField) . '</td>
			  </tr>'; */
		}
		if($isField){
			$open = '<we:ifNotFieldEmpty name="' . $name . '" type="' . $type . '">';
			$close = "</we:ifNotFieldEmpty>";
		} else {
			$open = '<we:ifNotVarEmpty name="' . $name . '" type="' . $type . '">';
			$close = "</we:ifNotVarEmpty>";
		}
		return '
<tr>
	<td style="width:100px;"><b>' . $name . '</b></td>
	<td style="width:300px">
		' . $open . '
		<we:listview type="multiobject" name="' . $name . '">
			<we:repeat>' . $this->getTemplTag($type, $name) . '</we:repeat>
		</we:listview>
		<we:else/>
			' . g_l('global', '[no_entries]') . '
		' . $close . '
	</td>
</tr>';
	}

	public function show(): string{
		$we_transaction = we_base_request::_(we_base_request::RAW, 'we_cmd', '', 3);
		$this->doc = we_contents_root::getSessionDocument($we_transaction);
		$tmpl = new we_object_createTemplate();

		$tmpl->we_new();
		$tmpl->Filename = $this->filename;
		$tmpl->Extension = ".tmpl";

		$tmpl->setParentID($this->pid);

		$this->usedIDs = [$this->doc->ID];

		$sort = $this->doc->getElementA('we_sort');

		$content = '<html>
	<head>
		<we:title></we:title>
		<we:description></we:description>
		<we:keywords></we:keywords>
	</head>
	<body>
		<table style="border1px solid black;width:400px">
';

		foreach(array_keys($sort) as $key){
			$wholename = $this->doc->getElementS('wholename' . $key);
			$name = $this->doc->getElementS($wholename);
			$type = $this->doc->getElementS($wholename . '#' . we_contents_object::ELEMENT_TYPE);

			$content .= $this->getTmplTableRow($type, $name);
		}

		$content .= '</table>';
		if($this->doc->ID){
			$content .= '
		<p>
		<we:listview type="object" classid="' . $this->doc->ID . '" rows="10">
			<we:repeat>
		<p><table style="border:1px solid black;width:400px">';


			foreach(array_keys($sort) as $key){
				$wholename = $this->doc->getElementS('wholename' . $key);
				$name = $this->doc->getElementS($wholename);
				$type = $this->doc->getElementS($wholename . '#' . we_contents_object::ELEMENT_TYPE);

				$content .= $this->getTmplTableRow($type, $name, true);
			}

			$content .= '</table></p>
			</we:repeat>
			<we:ifFound>
				<p><table class="default" style="width:400px;">
					<tr>
						<we:ifBack>
							<td><we:back>back</we:back></td>
						</we:ifBack>
						<we:ifNext>
							<td style="text-align:right"><we:next>next</we:next></td>
						</we:ifNext>
					</tr>
				</table></p>
			<we:else/>
				' . g_l('global', '[no_entries]') . '
			</we:ifFound>
		</we:listview>
';
		}
		$content .= '
	</body>
</html>';


//  $_SESSION['weS']["content"] is only used for generating a default template, it is
//  used only in WE_OBJECT_MODULE_PATH/we_object_createTemplatecmd.php
		$_SESSION['weS']['content'] = $content;

		$buttons = we_html_button::position_yes_no_cancel(
				we_html_button::create_button(we_html_button::SAVE, "javascript:if(document.we_form.we_" . $tmpl->Name . "_Filename.value != ''){ document.we_form.action=WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=we_object_class_createTemplateCmd';document.we_form.submit();}else{WE().util.showMessage(WE().consts.g_l.alert.input_file_name,WE().consts.message.WE_MESSAGE_ERROR, window);}"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
		);
		t_e('tmpl', $tmpl);
		$this->title = g_l('weClass', '[generateTemplate]');
		return parent::getPage(we_html_tools::dialogLayout($tmpl->formPath(), g_l('weClass', '[generateTemplate]'), $buttons) .
				we_html_element::hiddens(['SID' => $tmpl->Name,
					'we_cmd[3]' => $we_transaction,
					'we_cmd[2]' => we_base_request::_(we_base_request::RAW, 'we_cmd', '', 2)
		]));
	}

}
