<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4e4ff85098e9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-03 00:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_exim_searchPatterns{
	/** @var array<string,string[]> */
	public $doc_patterns = [
		'id' => [
			'/<(we:include\s*[^>]*\stype\s*=\s*[\"\']document[\"\'][^>]*\sid\s*=\s*[\"\'])(\d+)([\"\'][^>]*)>/si',
			'/<(we:include\s*[^>]*\sid\s*=\s*[\"\'])(\d+)([\"\']\stype\s*=\s*["\']document["\'][^>]*)>/si',
			//replace #WE:1223#
			'/(#WE:)(\d+)(#)/se'
		],
		'path' => [
			'/<(we:include\s*[^>]*\spath\s*=\s*[\"\'])([^\'\"> ? \\\]+)([\"\'][^>]*)>/si',
		]
	];

	/** @var array<string,string[]> */
	public $obj_patterns = [
		'id' => [
			'/<(we:object\s*[^>]*\sid\s*=\s*[\"\'])(\d+)([\"\'][^>]*)>/si',
			'/<(we:form\s*[^>]*\stype\s*=\s*[\"\']object[\"\'][^>]*\sid\s*=\s*[\"\'])(\d+)([\"\'][^>]*)>/si',
			'/<(we:form\s*[^>]*\sid\s*=\s*[\"\'])(\d+)([\"\'][^>]*\stype\s*=\s*[\"\']object[\"\'][^>]*)>/si',
		],
		'path' => []];

	/** @var string[] */
	public $class_patterns = [];

	/** @var string[] */
	public $ext_patterns = [];

	/** @var array<string,string[]> */
	public $wysiwyg_patterns = [
		'doc' => [
			'/(href\s*=\s*[\'\"]' . we_base_link::TYPE_INT_PREFIX . ')(\d+)([^\"\']*[\"\'])/si',
			'/(src\s*=\s*[\'\"]' . we_base_link::TYPE_INT_PREFIX . ')(\d+)([^\"\']*[\"\'])/si',
		],
		'obj' => ['/(href\s*=\s*[\"\']' . we_base_link::TYPE_OBJ_PREFIX . ')(\d+)([^\"\']*[\"\'])/si'
		]
	];

	/** @var string[] */
	public $navigation_patterns = [
		'/<(we:navigation\s*[^>]*\sid\s*=["\'])(\d+)(["\'][^>]*)>/si',
		'/<(we:navigation\s*[^>]*\sparentid\s*=["\'])(\d+)(["\'][^>]*)>/si'
	];

	/** @var string[] */
	public $thumbnail_patterns = [
		'/<(we:img\s*[^>]*\sthumbnail\s*=["\'])([^\'\"> ? \\\]+)(["\'][^>]*)>/si',
		'/<(we:field\s*[^>]*\sthumbnail\s*=["\'])([^\'\"> ? \\\]+)(["\'][^>]*)>/si',
		'/<(we:sessionField\s*[^>]*\sthumbnail\s*=["\'])([^\'\"> ? \\\]+)(["\'][^>]*)>/si',
		'/<(we:userInput\s*[^>]*\sthumbnail\s*=["\'])([^\'\"> ? \\\]+)(["\'][^>]*)>/si',
	];

	/** @var string[] */
	public $thumbnailid_patterns = [
		'/<(we:img\s*[^>]*\sthumbnailid\s*=["\'])([^\'\"> ? \\\]+)(["\'][^>]*)>/si',
		'/<(we:field\s*[^>]*\sthumbnailid\s*=["\'])([^\'\"> ? \\\]+)(["\'][^>]*)>/si',
		'/<(we:sessionField\s*[^>]*\sthumbnailid\s*=["\'])([^\'\"> ? \\\]+)(["\'][^>]*)>/si',
		'/<(we:userInput\s*[^>]*\sthumbnailid\s*=["\'])([^\'\"> ? \\\]+)(["\'][^>]*)>/si',
	];

	/** @var string[] */
	public $tmpl_patterns = [
		'/<(we:include\s*[^>]*\stype\s*=\s*["\']template["\'][^>]*\sid\s*=\s*["\'])(\d+)([\"\'][^>]*)>/si',
		'/<(we:include\s*[^>]*\sid\s*=\s*["\'])(\d+)([\"\']\s+type\s*=\s*["\']template["\'][^>]*)>/si',
		'/<(we:field\s*[^>]*\stid\s*=\s*[\"\'])(\d+)(["\'][^>]*)>/si',
		'/<(we:ifTemplate\s*[^>]*\sid\s*=\s*[\"\'])(\d+)(["\'][^>]*)>/si',
		'/<(we:ifNotTemplate\s*[^>]*\sid\s*=\s*[\"\'])(\d+)(["\'][^>]*)>/si',
	];
//	['match'=>[],				'use'=>''			]
	public $newTmplPattern = [
		'include' => [
			['match' => ['type' => 'template'], 'use' => 'id'],
		],
		'field' => [
			['match' => [], 'use' => 'tid']
		],
		'ifTemplate' => [
			['match' => [], 'use' => 'id']
		],
		'ifNotTemplate' => [
			['match' => [], 'use' => 'id']
		]
	];

	/** @var string[] */
	public $special_patterns = [
		'/<(we:include\s*[^>]*\sid\s*=\s*[\"\'])(\d+)(["\'][^>]*)>/si'
	];

	public const TEMPLATE_ID_PATTERNS = [
		'a' => ['id'],
		'addDelNewsletterEmail' => ['id', 'mailid'],
		'css' => ['id'],
		'delete' => ['pid'],
		'field' => ['id'],
		'form' => ['id', 'onsuccess', 'onerror', 'onmailerror', 'onrecipienterror'],
		'icon' => ['id'],
		'ifNotSelf' => ['id'],
		'ifSelf' => ['id'],
		'img' => ['id', 'startid', 'parentid'],
		'include' => ['id'],
		'js' => ['id'],
		'linkToSEEM' => ['id'],
		'linkToSeeMode' => ['id'],
		'listdir' => ['id'],
		'listview' => ['id', 'triggerid', 'workspaceID'],
		'printVersion' => ['triggerid'],
		'object' => ['triggerid'],
		'sessionLogout' => ['id'],
		'sendMail' => ['id'],
		'url' => ['id'],
		'video' => ['id', 'startid', 'parentid'],
		'write' => ['triggerid'],
	];

	public function __construct(){
		$this->wysiwyg_patterns = [
			'doc' => [
				'/(href\s*=\s*[\'\"]' . we_base_link::TYPE_INT_PREFIX . ')(\d+)([^\"\']*[\"\'])/si',
				'/(src\s*=\s*[\'\"]' . we_base_link::TYPE_INT_PREFIX . ')(\d+)([^\"\']*[\"\'])/si',
			],
			'obj' => ['/(href\s*=\s*[\"\']' . we_base_link::TYPE_OBJ_PREFIX . ')(\d+)([^\"\']*[\"\'])/si'
			]
		];

		foreach(self::TEMPLATE_ID_PATTERNS as $tag => $attribut){
			foreach($attribut as $attrib){
				$this->doc_patterns['id'][] = '/<(we:' . $tag . '\s*[^>]*\s' . $attrib . '\s*=\s*[\"\'])(\d+)(["\'][^>]*)>/si';
			}
		}

		// search for classes
		$pats = [
			'form' => ['classid'],
			'object' => ['classid'],
			'listview' => ['classid']
		];
		foreach($pats as $tag => $attribut){
			foreach($attribut as $attrib){
				$this->class_patterns[] = '/<(we:' . $tag . '\s*[^>]*\s' . $attrib . '\s*=\s*[\"\'])(\d+)(["\'][^>]*)>/si';
			}
		}

		// search for external files
		$pats = [
			'img' => ['src'],
			'a' => ['href'],
			'body' => ['background'],
			'table' => ['background'],
			'td' => ['background']
		];
		foreach($pats as $tag => $attribut){
			foreach($attribut as $attrib){
				$this->ext_patterns[] = '/<(' . $tag . '\s*[^>]*' . $attrib . '\s*=\s*[\"\'])([^\'\"> ? \\\]+)(["\'][^>]*)>/si';
			}
		}
	}

}
