<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d0033894a059 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-08-23 18:40 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_folder_objectFile extends we_folder_base{
	public const BACKUP_XML_TAG = 'we:classFolder';

	/** @var string */
	var $ClassPath = ''; //#4076

	/** @var int */
	var $RootfolderID = 0; //#4076

	/** @var we_doclist_searchObjectfiles */
	var $searchclass;

	/** @var string */
	var $Order = 'Path';

	/** @var string */
	//private $searchView = 'properties';

	/** @var string */
	var $Search = '';

	/** @var string */
	var $SearchField = '';

	// FIXME: use we_doclist_model to save all seach props but searchclass

	/** @var int */
	var $SearchStart = 0;

	/** @var int */
	protected $TableID = 0;

	/** @var string */
	var $WorkspacePath = '';

	/** @var int */
	var $WorkspaceID = 0;

	/** @var bool */
	protected $GreenOnly = false;

	/** @var int */
	var $rootDirID = 0;

	/** @var string database table in which the object is stored */
	public $Table = OBJECT_FILES_TABLE;

	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::FOLDER_OBJECTFILE;

	public function __construct(int $id = 0){
		parent::__construct($id);
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'SearchStart' => we_base_request::INT,
			'SearchField' => we_base_request::STRING,
			'Search' => we_base_request::STRING,
			'Order' => we_base_request::STRING,
			'searchclass' => we_base_request::STRING,
			'TriggerID' => we_base_request::INT,
			'urlMap' => we_base_request::STRING,
			'TableID' => we_base_request::INT,
			'WorkspacePath' => we_base_request::STRING,
			'WorkspaceID' => we_base_request::INT,
			'GreenOnly' => we_base_request::BOOL,
			'rootDirID' => we_base_request::INT,
		]);

		if(isWE()){
			if($this->ID){
				$this->EditPageNrs = array_merge($this->EditPageNrs, [
					we_base_constants::WE_EDITPAGE_PROPERTIES,
					we_base_constants::WE_EDITPAGE_FIELDS,
					we_base_constants::WE_EDITPAGE_INFO]);

				if($this->doclistModel){
					$this->doclistModel->setTableID($this->TableID);
				}


				if(!$this->searchclass && isset($_SESSION['weS']['we_objectSearch'][$this->ID])){
					$temp = $_SESSION['weS']['we_objectSearch'][$this->ID];
					$this->searchclass = $temp['Searchclass'];
					$this->EditPageNr = $temp['EditPageNr'];

					// use these on search_model
					$this->SearchStart = $temp['SearchStart'];
					$this->GreenOnly = $temp['GreenOnly'];
					$this->Order = $temp['Order'];
				}

				// FIXME: do we need searchclass here?
				if($this->doclistModel){
					$this->searchclass = $this->searchclass ?: new we_doclist_searchObjectfiles(new we_search_view($this->doclistModel));
				}
			}
			$this->EditPageNr = $this->EditPageNr ?: we_base_constants::WE_EDITPAGE_PROPERTIES;
			$this->setClassProp();
		}
	}

	/**
	 *
	 * @return bool
	 */
	public function isClassFolder(): bool{
		return (bool) $this->ParentID;
	}

	/**
	 *
	 * @param string[] $keep
	 * @return void
	 */
	public function makeSameNew(array $keep = []): void{
		parent::makeSameNew(array_merge($keep, ['TableID', 'TriggerID', 'ClassPath', 'RootfolderID', 'ParentID', 'Table']));
	}

	function adjustEditPageNr(): void{
		if(!isWE()){
			return;
		}
		if($this->ID){
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_FIELDS, we_base_constants::WE_EDITPAGE_INFO]);
		}
	}

	private function setClassProp(): void{
		if(!$this->Path){
			return;
		}
		$sp = explode('/', $this->Path);
		$this->ClassPath = '/' . $sp[1];

		['root' => $this->RootfolderID, 'ID' => $this->TableID] = (getHash('SELECT IFNULL(of.ID,0) AS root,o.ID FROM ' . OBJECT_TABLE . ' o LEFT JOIN ' . OBJECT_FILES_TABLE . ' `of` ON (of.TableID=o.ID AND of.IsClassFolder=1) WHERE o.Path="' . $this->DB_WE->escape($this->ClassPath) . '"') ?: [
			'root' => 0, 'ID' => 0]);
	}

	public function we_rewrite(): bool{
		return $this->we_save(false, true);
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if(!$this->wasUpdate){
			$this->ModifierID = we_users_user::getID();
			$this->CreatorID = $this->CreatorID ?: $this->ModifierID;
			$this->TableID = !$this->TableID && $this->ParentID ? fI('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $this->ParentID . ' AND IsFolder=1') : $this->TableID;
		}

		parent::we_save($resave, $skipHook);

		return true;
	}

	public function initByPath(string $path, string $tblName = OBJECT_FILES_TABLE, bool $skipHook = false): bool{
		if(($id = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Path="' . $path . '" AND IsFolder=1'))){
			$this->initByID($id);
			return true;
		}
		## Folder does not exist, so we have to create it (if user has permissons to create folders)
		$spl = explode('/', $path);
		$folderName = (string) (array_pop($spl) ?? '');
		$p = [];
		$anz = count($spl);
		$last_pid = 0;
		for($i = 0; $i < $anz; $i++){
			$p[] = array_shift($spl);
			$pa = $this->DB_WE->escape(implode('/', $p));
			if($pa){
				if(($pid = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Path="' . $pa . '"'))){
					$last_pid = $pid;
				} else {
					$folder = new self();
					$folder->we_new();
					$folder->Table = OBJECT_FILES_TABLE;
					$folder->ParentID = $last_pid;
					$folder->Filename = (string) $p[$i];
					$folder->Path = $pa;
					$folder->save($skipHook);
					$last_pid = $folder->ID;
				}
			}
		}
		$this->we_new();
		$this->ParentID = $last_pid;
		$this->ParentPath = id2path($last_pid, $this->Table);
		$this->Filename = $folderName;
		$this->Path = $path;
		//#4076
		$this->setClassProp();

		return $this->save(false, $skipHook);
	}

	protected function isSaveDirinDir(): bool{
		return $this->isClassFolder();
	}

	/* must be called from the editor-script. Returns a filename which has to be included from the global-Script */

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			//no break
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_FIELDS:
				return new we_editor_doclist($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_WEBUSER:
				return new we_editor_weDocumentCustomerFilter($this, $jsCmd);
		}
	}

	protected function formCopyDocument(): string{
		$idname = 'we_' . $this->Name . '_CopyID';
		$parents = array_merge([0, $this->ID], we_base_util::getParentIDs(FILE_TABLE, $this->ID));
		$this->setClassProp();
		if($this->ID){
			$disabled = false;
			$disabledNote = '';
		} else {
			$disabled = true;
			$disabledNote = ' ' . g_l('weClass', '[availableAfterSave]');
		}
//"javascript:we_cmd('we_selector_directory', document.we_form.elements['" . $idname . "'].value, '" . $this->Table . "', '" . $idname . "', '', 'copyFolderCheck," . $this->ID . "," . $this->Table . "," . implode(',', $parents) . "')" :
		$but = we_html_button::create_button(we_html_button::SELECT, $this->ID ? "javascript:we_cmd('we_selector_directory',{id:document.forms[0].elements['" . $idname . "'].value, table:'" . OBJECT_FILES_TABLE . "', targetID:'" . $idname . "', jsCmd:'copyFolderCheck," . $this->ID . "," . OBJECT_FILES_TABLE . "," . implode(',', $parents) . "',rootDirID:" . $this->RootfolderID . "});" : "javascript:WE().util.showMessage(WE().consts.g_l.alert.copy_folders_no_id,WE().consts.message.WE_MESSAGE_ERROR, window);", '', '', $disabled);

		return
			we_html_tools::alertAttentionBox(g_l('weClass', '[copy_owners_expl]') . $disabledNote, we_html_tools::TYPE_INFO, 0, false) . we_html_element::hidden($idname, $this->CopyID) . we_html_element::br() .
			$but;
	}

	function saveInSession(string $transaction): void{
		parent::saveInSession($transaction);

		if(!isset($_SESSION['weS']['we_objectSearch'])){
			$_SESSION['weS']['we_objectSearch'] = [];
		}
		$_SESSION['weS']['we_objectSearch'][$this->ID] = [
			'Searchclass' => $this->searchclass,
			'SearchStart' => $this->SearchStart,
			'GreenOnly' => $this->GreenOnly,
			'Order' => $this->Order,
			'EditPageNr' => $this->EditPageNr,
		];
	}

	function setObjectProperty(array $IDs, string $property = '', bool $value = false): void{
		$this->setClassProp();
		if(!$IDs){
			return;
		}

		$workspaces = '';
		switch($property){
			case 'IsSearchable':
				$value = intval($value);
				$set = [$property => $value];
				break;
			case 'TriggerID':
				$value = fI('SELECT DefaultTriggerID FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->TableID);
				$set = [$property => $value];
				break;
			case 'Workspaces':
				$workspaces = fS('SELECT Workspaces FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->TableID);
				$set = ['Workspaces' => $workspaces];
				break;
			default:
				return;
		}

		$whereRestrictOwners = we_users_util::makeOwnersSql(false);
		$this->DB_WE->query('UPDATE ' . OBJECT_FILES_TABLE . ' `of` SET ' . we_database_base::arraySetter($set) . ' WHERE of.ID IN(' . implode(',', $IDs) . ') AND of.IsFolder=0' . $whereRestrictOwners);

		$affected = 'SELECT ID FROM ' . OBJECT_FILES_TABLE . ' `of` WHERE of.ID IN(' . implode(',', $IDs) . ') AND of.IsFolder=0' . $whereRestrictOwners;
		//change tblIndex
		switch($property){
			case 'IsSearchable':
				$value = intval($value);
				if($value){
					$docs = DB_WE::$inst->getAllq($affected, true);
					'@phan-var int[] $docs';
					foreach($docs as $id){
						$obj = new we_contents_objectFile($id);
						$obj->insertAtIndex();
					}
				} else {
					$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID IN(' . $affected . ')');
				}
				return;
			case 'Workspaces':
				//delete all unneeded workspaces
				if($workspaces){
					$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID IN(' . $affected . ') AND WorkspaceID NOT IN(' . $workspaces . ')');

//we need to add all searchable objects to tblindex
					$docs = DB_WE::$inst->getAllq($affected . ' AND IsSearchable=1', true);
					'@phan-var int[] $docs';
					foreach($docs as $id){
						$obj = new we_contents_objectFile($id);
						$obj->insertAtIndex();
					}
				}
				return;
		}

		return;
	}

	function publishObjects(array $IDs, bool $publish = true): void{
		$this->setClassProp();
		if(!$IDs){
			return;
		}

		foreach($IDs as $ofid){
			$ofid = (int) $ofid;
			if(!we_base_permission::checkIfRestrictUserIsAllowed($ofid, OBJECT_FILES_TABLE)){
				continue;
			}
			$obj = new we_contents_objectFile();
			$obj->initByID($ofid, '', self::LOAD_TEMP_DB);
			if($publish){
				//update url
				$obj->setUrl();
			}

			$update = ($publish ? $obj->we_publish() : $obj->we_unpublish());
			if($update){
				//write some answer message
			}
		}
	}

	protected function isPathNotValid(): bool{
		return $this->isClassFolder() ? false : (parent::isPathNotValid() || $this->ParentID == 0 || $this->ParentPath === '/');
	}

	public function updateEntry(int $tableID, string $filename, int $parentID, int $ID = 0): bool{
		if($ID){
			$this->initByID($ID);
		}
		$this->Filename = $filename;
		$this->setParentID($parentID);
		$this->TableID = $tableID;
		$ret = $this->we_save(true);
		if(!$ret){
			return false;
		}
		$this->modifyChildrenPath();
		return true;
	}

	protected function hasCopyPerm(): bool{
		return we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_COPY]);
	}

}
