<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: ead093ffe1e6 $
 * $Author: Lukas Imhof $
 * $Date: 2020-10-01 21:52 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_wizardCSV extends we_import_wizard{

	protected function getWizBody(): string{
		$formAttribs = [
			'name' => we_base_constants::WE_FORM,
			'enctype' => 'multipart/form-data'
		];

		$jsCmd = new we_base_jsCmd();
		$hiddens = [
			'pnt' => 'wizbody',
			'type' => we_exim_ExIm::TYPE_CSV,
			'v[type]' => we_exim_ExIm::TYPE_CSV,
			'step' => (string) $this->step,
			'mode' => (string) $this->mode,
			'button_state' => '0'
		];
		switch($this->step){
			case 2:
				$this->fileUploaderResp = new we_fileupload_resp_base();
				break;
			default:
				$this->fileUploader = new we_fileupload_ui_base('uploaded_csv_file', [], we_base_ContentTypes::getExtension(we_base_ContentTypes::TEXT));
				$this->fileUploader->setCmd('cmdFileselectOnclick', 'fileupload_doOnFileSelect');
				$this->fileUploader->setGenericFileName(TEMP_DIR . 'we_csv_' . we_fileupload::REPLACE_BY_UNIQUEID . '.csv');
		}
		switch($this->step){
			default:
			case 1:
				$content = $this->getStep1();
				break;
			case 2:
				$content = $this->getStep2($jsCmd);
				break;
			case 3:
				$content = $this->getStep3();
				break;
		}
		$doOnLoad = !we_base_request::bool('noload');
		return we_html_tools::getHtmlTop('', ($this->fileUploader ? $this->fileUploader->getCss() . $this->fileUploader->getJs() : '') .
				we_html_element::jsScript(JS_DIR . 'import_wizardWizbody.js') .
				$jsCmd->getCmds(), we_html_element::body(['class' => 'weDialogBody',
					'onload' => $doOnLoad ? "parent.wiz_next('wizbusy', WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=import&pnt=wizbusy&mode=" . $this->mode . "&type=" . we_exim_ExIm::TYPE_CSV . "'); self.focus();" : ""
					], we_html_element::form($formAttribs, we_html_element::hiddens($hiddens) .
						$content
					)
				)
		);
	}

	private function getStep1(): string{
		$v = we_base_request::stringList('v');

		$v['import_type'] = $v['import_type'] ?? self::TYPE_DOCUMENT;

		$importFromButton = (we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT])) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', 'v[fserver]', 'all_Types', document.we_form.elements['v[fserver]'].value)") : '';
		$inputLServer = we_html_tools::textInput('v[fserver]', ($v['fserver'] ?? '/'), 255, ['readonly' => 'readonly', 'onclick' => "WE().util.dialogTop(window).setFormField('v[rdofloc]', true, 'radio', 0);"], "text", '300px');
		$importFromServer = we_html_tools::formElementTable('', $inputLServer, $importFromButton);

		$inputLLocal = $this->fileUploader->getHTML();
		$importFromLocal = we_html_tools::formElementTable('', $inputLLocal);
		$rdoLServer = we_html_forms::radiobuttonV('lServer', $v['rdofloc'], 'v[rdofloc]', g_l('import', '[fileselect_server]'));
		$rdoLLocal = we_html_forms::radiobuttonV('lLocal', $v['rdofloc'], 'v[rdofloc]', g_l('import', '[fileselect_local]'));
		$tblRow = 0;
		$importLocs = (new we_html_table(['class' => 'default'], 6, 1))
			->setCol($tblRow++, 0, $rdoLServer)
			->setCol($tblRow++, 0, $importFromServer)
			->setCol($tblRow++, 0, $rdoLLocal, ['style' => 'padding-top:4px;'])
			// FIXME: still need condition?
			//$importLocs->setCol($tblRow++, 0, $this->fileUploader->getHtmlAlertBoxes());
			->setCol($tblRow++, 0, $importFromLocal);

		$iptDel = we_html_tools::textInput('v[csv_seperator]', (isset($v['csv_seperator']) ? (($v['csv_seperator'] != '') ? $v['csv_seperator'] : ' ') : ';'), 2, [], 'text', '30px');
		$fldDel = (new we_html_select(['name' => 'v[sct_csv_seperator]', 'class' => 'weSelect', 'onchange' => "this.form.elements['v[csv_seperator]'].value=this.options[this.selectedIndex].innerHTML.substr(0,2);this.selectedIndex=options[0];",
				'style' => "width: 130px"
				])) // FIXME: register change listener
			->addOptions(we_exim_ExportCSV::getCSVDelimeters());
		if(isset($v['sct_csv_seperator'])){
			$fldDel->selectOption($v['sct_csv_seperator']);
		}

		$charSet = (new we_html_select(['name' => 'v[file_format]', 'class' => 'weSelect',]))
			->addOptions([
			we_exim_ExportCSV::LINEEND_WINDOWS => 'Windows',
			we_exim_ExportCSV::LINEEND_UNIX => 'Unix',
			we_exim_ExportCSV::LINEEND_MAC => 'Mac'
		]);
		if(isset($v['file_format'])){
			$charSet->selectOption($v['file_format']);
		}

		$txtDel = (new we_html_select(['name' => 'v[csv_enclosed]', 'class' => 'weSelect', 'style' => 'width: 300px']))
			->addOptions([
			'double_quote' => g_l('import', '[double_quote]'),
			'single_quote' => g_l('import', '[single_quote]'),
			'none' => g_l('import', '[none]')
		]);
		if(isset($v['csv_enclosed'])){
			$txtDel->selectOption($v['csv_enclosed']);
		}

		$rowDef = we_html_forms::checkbox('', (bool) ($v['csv_fieldnames'] ?? true), 'checkbox_fieldnames', g_l('import', '[contains]'), true, 'defaultfont', "this.form.elements['v[csv_fieldnames]'].value=this.checked ? 1 : 0;");

		$csvSettings = (new we_html_table(['class' => 'default withSpace'], 4, 1))
			->setCol(0, 0, g_l('import', '[file_format]') . we_html_element::br(2) . $charSet->getHtml())
			->setCol(1, 0, g_l('import', '[field_delimiter]') . we_html_element::br(2) . $iptDel . ' ' . $fldDel->getHtml())
			->setCol(2, 0, g_l('import', '[text_delimiter]') . we_html_element::br(2) . $txtDel->getHtml())
			->setCol(3, 0, $rowDef);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('import', '[import]'), $importLocs->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED))
			->add((new we_html_multiIconBoxContentLine(g_l('import', '[field_options]'), $csvSettings->getHTML()))
			->space(we_html_multiIconBox::SPACE_MED)
			->noline());

		return we_html_element::hiddens([
				'v[csv_fieldnames]' => $v['csv_fieldnames'] ?? 1,
				'v[import_from]' => ($v['import_from'] ?? ''),
				'v[csv_escaped]' => $v['csv_escaped'] ?? '',
				'v[collision]' => $v['collision'] ?? we_exim_ExIm::COLLISION_RENAME,
				'v[csv_terminated]' => $v['csv_terminated'] ?? '',
				'v[btnState_next]' => 'enabled',
				'v[btnState_back]' => 'enabled'
			]) .
				(new we_html_multiIconBox($parts, 'csv'))
				->marginLeft()
				->headline(g_l('import', '[csv_import]'))
				->get();
	}

	private function getStep2(we_base_jsCmd $jsCmd): string{
		$DB_WE = we_database_base::$inst;
		$v = we_base_request::stringList('v');
		$upload_error = false;

		$btnStateNext = $btnStateBack = 'enabled';

		if($v['rdofloc'] === 'lLocal'){
			try{
				$v['import_from'] = $this->fileUploaderResp->commitUploadedFile();
			} catch (we_exception_file $e){
				$upload_error = $e->getMessage();
			}
		} else {
			$realPath = realpath(WE_DOCUMENT_ROOT . $v['import_from']) ?: '';
			if(strpos($realPath, WE_DOCUMENT_ROOT) === false){
				t_e('warning', 'Acess outside document_root forbidden!', $realPath);
			} else {
				$contents = we_base_file::load($realPath, 'r');
				$v['import_from'] = TEMP_DIR . 'we_csv_' . we_base_file::getUniqueId() . '.csv';
				$replacement = str_replace("\r", "\n", $contents);
				we_base_file::save(WE_DOCUMENT_ROOT . $v['import_from'], $replacement, 'w+');
			}
		}

		if(isset($v['docType']) && $v['docType'] != -1 && we_base_request::bool('doctypeChanged')){
			$values = getHash('SELECT ParentID,Extension,IsDynamic,Category FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.ID=' . intval($v['docType']));
			$v['store_to_id'] = $values['ParentID'];

			$v['store_to_path'] = id2path($v['store_to_id']);
			$v['we_Extension'] = $values['Extension'];
			$v['is_dynamic'] = $values['IsDynamic'];
			$v['docCategories'] = $values['Category'];
		}
		$hdns = we_html_element::hiddens([
				'v[mode]' => ($v['mode'] ?? ''),
				'v[import_from]' => $v['import_from'],
				'v[collision]' => $v['collision'],
				'v[rdofloc]' => $v['rdofloc'],
				'v[fserver]' => $v['fserver'],
				'v[csv_fieldnames]' => $v['csv_fieldnames'],
				'v[csv_seperator]' => trim($v['csv_seperator']),
				'v[csv_enclosed]' => $v['csv_enclosed'],
				'v[csv_escaped]' => $v['csv_escaped'],
				'v[csv_terminated]' => $v['csv_terminated'],
				'v[docCategories]' => ($v['docCategories'] ?? ''),
				'v[objCategories]' => ($v['objCategories'] ?? ''),
				//rray('name' => 'v[store_to_id]', 'value' => (isset($v['store_to_id']) ? $v['store_to_id'] : 0))).
				'v[we_TemplateID]' => ($v['we_TemplateID'] ?? '0'),
				'v[is_dynamic]' => ($v['is_dynamic'] ?? '0'),
				'doctypeChanged' => '0',
				'v[file_format]' => $v['file_format'],
				(we_modules_info::isActive(we_modules_info::OBJECT) ? '' : 'v[import_type]') => self::TYPE_DOCUMENT,
		]);

		$v['import_type'] = $v['import_type'] ?? self::TYPE_DOCUMENT;

		$optid = 0;
		$DTselect = (new we_html_select(['name' => 'v[docType]',
				'class' => 'weSelect',
				'onclick' => we_modules_info::isActive(we_modules_info::OBJECT) ? "WE().util.dialogTop(window).setFormField('v[import_type]', true, 'radio', 0);" : '',
				'onchange' => "WE().util.dialogTop(window).setFormField('doctypeChanged', 1, 'hidden'); WE().util.dialogTop(window).weChangeDocType(this);",
				'style' => 'width: 300px'
				]))
			->insertOption($optid, '-1', g_l('import', '[none]'));

		$v['docType'] = $v['docType'] ?? -1;
		$dts = DB_WE::$inst->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.IsFolder=0 ORDER BY dt.Text', false);
		'@phan-var array<int,string> $dts';
		foreach($dts as $id => $dt){
			$optid++;
			$DTselect->insertOption($optid, (string) $id, $dt);
			if($v['docType'] == $id){
				$DTselect->selectOption($id);
			}
		}

		$doctypeElement = we_html_tools::formElementTable(g_l('import', '[doctype]'), $DTselect->getHTML());

		/*		 * * templateElement *************************************************** */
		/* $ueberschrift = (we_base_permission::hasPerm([we_base_perm::CAN_SEE_TEMPLATES]) ?
		  '<a href="javascript:goTemplate(document.we_form.elements[\'' . $idname . '\'].value)">' . g_l('import', '[template]') . '</a>' :
		  g_l('import', '[template]')); */

		$myid = $v['we_TemplateID'] ?? 0;

		$weSuggest = we_gui_suggest::getInstance();

		$TPLselect = new we_html_select([
			'name' => 'docTypeTemplateId',
			'class' => 'weSelect',
			'onclick' => "WE().util.dialogTop(window).setFormField('v[import_type]', true, 'radio', 0);",
			'style' => "width: 300px"
		]);

		if($v['docType'] != -1){
			$displayDocType = $displayNoDocType = '';
			$foo = fS('SELECT Templates FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.ID=' . intval($v['docType']));
			$TPLselect->addOptions(id2pathA(we_base_request::filterIntList($foo), TEMPLATES_TABLE));
			if(!empty($v['we_TemplateID'])){
				$TPLselect->selectOption($v['we_TemplateID']);
			}
		} else {
			$displayDocType = 'display:none';
			$displayNoDocType = 'display:block';
		}
		$weSuggest->setAcId('TmplPath')
			->setContentType([we_base_ContentTypes::FOLDER_TEMPLATE, we_base_ContentTypes::TEMPLATE])
			->setInput('v[we_TemplateName]', (string) ($v['we_TemplateName'] ?? ''), ['onfocus' => "self.document.we_form.elements['v[import_type]'][0].checked=true;"])
			->setMaxResults(10)
			->setResult('noDocTypeTemplateId', (string) $myid)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(TEMPLATES_TABLE)
			->setWidth(300)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:top.wizbody.document.we_form.elements.noDocTypeTemplateId.value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'noDocTypeTemplateId',targetName:'v[we_TemplateName]',jsCmd:'reload_editpage',filter:WE().consts.contentTypes.TEMPLATE,open_doc:true})"))
			->setLabel(g_l('import', '[template]'));

		$templateElement = "<div id='docTypeLayer' style='" . $displayDocType . "'>" . we_html_tools::formElementTable(g_l('import', '[template]'), $TPLselect->getHTML()) . "</div>
<div id='noDocTypeLayer' style='" . $displayNoDocType . "'>" . $weSuggest->getHTML() . '</div>';

		$weSuggest->setAcId('DirPath')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
			->setInput('v[store_to_path]', ($v['store_to_path'] ?? '/'), ['onfocus' => "self.document.we_form.elements['v[import_type]'][0].checked=true;"])
			->setMaxResults(10)
			->setRequired(true)
			->setResult('v[store_to_id]', (string) ($v['store_to_id'] ?? 0))
			->setSelector(we_gui_suggest::DirSelector)
			->setWidth(300)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:WE().util.dialogTop(window).wizbody.document.we_form.elements['v[store_to_id]'].value,table:'" . FILE_TABLE . "',targetID:'v[store_to_id]',targetName:'v[store_to_path]',rootDirID:0})"))
			->setLabel(g_l('import', '[import_dir]'));

		$storeTo = $weSuggest->getHTML();

		$seaPu = (new we_html_table(['class' => 'default'], 2, 1))
			->setCol(1, 0, we_html_forms::checkboxWithHidden(!empty($v['doc_search']), 'v[doc_search]', g_l('weClass', '[IsSearchable]'), false, 'defaultfont'))
			->setCol(0, 0, we_html_forms::checkboxWithHidden((bool) ($v['doc_publish'] ?? true), 'v[doc_publish]', g_l('button', '[publish][value]'), false, 'defaultfont'));

		$docCategories = (we_modules_info::isActive(we_modules_info::CATEGORY) ? $this->formCategory2($jsCmd, 'doc', $v['docCategories'] ? we_base_request::filterIntList($v['objCategories']) : [
			]) : '');

		$radioDocs = we_html_forms::radiobuttonV(self::TYPE_DOCUMENT, $v['import_type'], "v[import_type]", g_l('import', '[documents]'));
		$radioObjs = we_html_forms::radiobuttonV(self::TYPE_OBJECT, $v['import_type'], 'v[import_type]', g_l('import', '[objects]'), true, '', "self.document.we_form.elements['v[store_to_path]'].value='/'; WE().layout.weSuggest.checkRequired(window,self.document.we_form.elements['v[store_to_path]'].id); if(self.document.we_form.elements['v[we_TemplateName]']!==undefined) { self.document.we_form.elements['v[we_TemplateName]'].value=''; WE().layout.weSuggest.checkRequired(window,self.document.we_form.elements['v[we_TemplateName]'].id); }", (!we_modules_info::isActive(we_modules_info::OBJECT)));

		$optid = $firstID = 0;
		$first = '';
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$v['classID'] = $v['classID'] ?? -1;
			$CLselect = new we_html_select([
				'id' => 'classID',
				'name' => 'v[classID]',
				'class' => 'weSelect',
				'onclick' => "WE().util.dialogTop(window).setFormField('v[import_type]', true, 'radio', 1);",
				'onchange' => "WE().util.dialogTop(window).onChangeSelectObject(this);",
				'style' => 'width: 150px'
			]);
			$ac = implode(',', we_users_util::getAllowedClasses());
			if($ac){
				$recs = $DB_WE->getAllq('SELECT o.ID,o.Text,of.ID AS FID FROM ' . OBJECT_TABLE . ' o LEFT JOIN ' . OBJECT_FILES_TABLE . ' `of` ON o.Text=of.Text WHERE o.ID IN(' . $ac . ') AND of.IsFolder=1 AND of.ParentID=0 ORDER BY o.Text');
				foreach($recs as $rec){
					if(!$optid){
						$first = '/' . $rec['Text'];
						$firstID = $rec['FID'];
					}
					$optid++;
					$CLselect->insertOption($optid, $rec['ID'] . '_' . $rec['FID'], $rec['Text']);
					if($rec['ID'] == $v['classID']){
						$CLselect->selectOption($rec['ID']);
					}
				}
			} else {
				$CLselect->insertOption($optid, '-1', g_l('import', '[none]'));
			}

			$objClass = (new we_html_table(['class' => 'default'], 1, 2))
				->setCol(0, 0, g_l('import', '[class]'), ['style' => 'vertical-align:top;width:130px;', 'class' => "defaultfont lowContrast"])
				->setCol(0, 1, $CLselect->getHTML(), ['style' => 'width:150px;']);

			$weSuggest->setAcId('ObjPath')
				->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE])
				->setInput('v[obj_path]', (string) ($v['obj_path'] ?? $first), ['onfocus' => "self.document.we_form.elements['v[import_type]'][1].checked=true;"])
				->setMaxResults(10)
				->setRequired(true)
				->setResult('v[obj_path_id]', (string) ($v['obj_path_id'] ?? ($firstID ?? 0)))
				->setSelector(we_gui_suggest::DirSelector)
				->setTable(OBJECT_FILES_TABLE)
				->setWidth(300)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['v[obj_path]'].value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'v[obj_path_id]',targetName:'v[obj_path]'})")) //must be jsCmd: document.we_form.elements['v[classID]'].value.split('_')[1]
				->setLabel(g_l('import', '[import_dir]'));

			$objStoreTo = $weSuggest->getHTML();

			$objCats = (new we_html_table(['class' => 'default'], 1, 2))
				->setCol(0, 0, g_l('import', '[categories]'), ['style' => 'vertical-align:top;width:130px;', 'class' => "defaultfont lowContrast"])
				->setCol(0, 1, $this->formCategory2($jsCmd, 'obj', $v['objCategories'] ? we_base_request::filterIntList($v['objCategories']) : [
				]), ['style' => 'width:150px;']);

			$objSeaPu = (new we_html_table(['class' => 'default'], 2, 1))
				->setCol(1, 0, we_html_forms::checkboxWithHidden(!empty($v['obj_search']), 'v[obj_search]', g_l('weClass', '[IsSearchable]'), false, 'defaultfont'))
				->setCol(0, 0, we_html_forms::checkboxWithHidden((bool) ($v['obj_publish'] ?? true), 'v[obj_publish]', g_l('button', '[publish][value]'), false, 'defaultfont'));

			$objCategories = (new we_html_table(['class' => 'default withBigSpace'], 3, 2))
				->setCol(0, 0, $radioObjs, ['colspan' => '3', 'style' => 'width:50px;'])
				->setCol(1, 1, $objClass->getHTML())
				->setCol(2, 1, $objCats->getHTML())
				->getHtml();
		} else {
			$CLselect = $objSeaPu = null;
			$objCategories = $objStoreTo = '';
		}

		$specifyDoc = (new we_html_table(['class' => 'default'], 1, 2))
			->setCol(0, 1, we_html_forms::checkbox('3', (bool) ($v['is_dynamic'] ?? false), "chbxIsDynamic", g_l('import', '[isDynamic]'), true, "defaultfont", "this.form.elements['v[is_dynamic]'].value=this.checked? 1 : 0; WE().util.dialogTop(window).switchExt();"), [
				'style' => 'vertical-align:bottom;'])
			->setCol(0, 0, we_html_tools::formElementTable(g_l('import', '[extension]'), we_html_tools::getExtensionPopup("v[we_Extension]", ($v['we_Extension'] ?? '.html'), we_base_ContentTypes::getExtension(we_base_ContentTypes::WEDOCUMENT), 100)), [
			'style' => 'padding-right:20px;']);
		$parts = new we_html_multiIconBoxContent();
		if((file_exists(WE_DOCUMENT_ROOT . $v['import_from']) && is_readable(WE_DOCUMENT_ROOT . $v['import_from']))){
			$parts
				->add((new we_html_multiIconBoxContentLine((we_modules_info::isActive(we_modules_info::OBJECT)) ? $radioDocs : g_l('import', '[documents]'), $doctypeElement .
						$templateElement .
						$storeTo .
						$specifyDoc->getHTML() .
						$seaPu->getHtml() .
						($docCategories ? we_html_tools::formElementTable(g_l('import', '[categories]'), $docCategories) : '')))
					->space(we_html_multiIconBox::SPACE_MED)
					->noline())
				->add(we_modules_info::isActive(we_modules_info::OBJECT) ?
						(new we_html_multiIconBoxContentLine($radioObjs, we_html_tools::formElementTable(g_l('import', '[class]'), $CLselect->getHTML()) .
							$objStoreTo .
							$objSeaPu->getHtml() .
							we_html_tools::formElementTable(g_l('import', '[categories]'), $objCategories)))
						->space(we_html_multiIconBox::SPACE_MED)
						->noline() : null);
		} elseif(!file_exists(WE_DOCUMENT_ROOT . $v['import_from'])){
			$parts
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('import', '[file_exists]') . WE_DOCUMENT_ROOT . $v['import_from'] . $upload_error, we_html_tools::TYPE_ALERT, 530)))
					->noline());
			$btnStateNext = 'disabled';
			$btnStateBack = 'enabled';
		} else if(!is_readable(WE_DOCUMENT_ROOT . $v['import_from'])){
			$parts
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('import', '[file_readable]') . $upload_error, we_html_tools::TYPE_ALERT, 530)))
					->noline());
			$btnStateNext = 'disabled';
		}

		return $hdns . we_html_element::hiddens(['v[btnState_next]' => $btnStateNext, 'v[btnState_back]' => $btnStateBack]) .
				(new we_html_multiIconBox($parts, 'csv'))
				->marginLeft()
				->headline(g_l('import', '[csv_import]'))
				->get();
	}

	private function getStep3(): string{
		$tid = we_base_request::int('v', 0, 'we_TemplateID');
		$tname = we_base_request::_(we_base_request::FILE, 'v', '', 'we_TemplateName');
		if($tname && !$tid){
			$_REQUEST['v']['we_TemplateID'] = path_to_id($tname, TEMPLATES_TABLE);
		}

		$v = we_base_request::stringList('v');

		$records = we_base_request::_(we_base_request::RAW, 'records', []);
		$we_flds = we_base_request::stringList('we_flds');
		$attrs = we_base_request::stringList('attrs');

		$csvFile = WE_DOCUMENT_ROOT . we_base_request::_(we_base_request::FILE, 'v', '', "import_from");
		if(file_exists($csvFile) && is_readable($csvFile)){
			$data = we_base_file::loadPart($csvFile);
			$encoding = mb_detect_encoding($data, 'UTF-8,ISO-8859-1,ISO-8859-15') ?: 'ISO-8859-15';
			if($encoding !== 'UTF-8'){
				we_base_file::save($csvFile, mb_convert_encoding(we_base_file::load($csvFile), 'UTF-8', $encoding . ',UTF-8') ?: '');
			}
		} else {
			$data = $encoding = '';
		}

		$hdns = $this->getHdns('v', we_base_request::stringList('v')) .
			($records ? $this->getHdns('records', $records) : '') .
			($we_flds ? $this->getHdns('we_flds', $we_flds) : '') .
			($attrs ? $this->getHdns('attrs', $attrs) : '') .
			we_html_element::hiddens([
				'v[startCSV]' => (string) (we_base_request::bool('v', false, 'startCSV')),
				'v[cid]' => (string) -2,
				'v[encoding]' => $encoding,
				'v[pfx_fn]' => we_base_request::string('v', '', 'pfx_fn')]) . /* rdo_timestamp is a string: 'GMT', 'UNIX' or 'Format' */
			(($tm = we_base_request::string('v', '', 'rdo_timestamp')) ? we_html_element::hidden('v[sTimeStamp]', $tm) : '');

		$records = $dateFields = [];

		if(we_base_request::string('v', '', 'import_type') === self::TYPE_DOCUMENT){
			$templateCode = fS('SELECT c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND c.DID=' . we_base_request::int('v', 0, 'we_TemplateID') . ' AND c.nHash=x\'' . md5(we_contents_template::FULLCODE) . '\'');
			$tp = new we_weTag_parser($templateCode);

			foreach($tp->getTagsWithAttributes() as $tag){
				if(empty($tag['attribs']['name'])){
					continue;
				}
				switch($tag['name']){
					case 'field':
					case 'var':
						continue 2;
					// tags with text content, links and hrefs
					case 'input':
						if(!empty($tag['attribs']['type']) && $tag['attribs']['type'] === 'date'){
							$dateFields[] = $tag['attribs']['name'];
						}
					case 'textarea':
					case 'href':
					case 'link':
						$records[] = $tag['attribs']['name'];
						break;
				}
			}
			$records = array_unique($records) ?: ['Title', 'Description', 'Keywords'];
		} else {
			[$class] = explode('_', we_base_request::string('v', '0', 'classID'));
			$classFields = self::getClassFields((int) $class);
			foreach($classFields as $classField){
				if(self::isTextField($classField['type']) || self::isNumericField($classField['type']) || self::isDateField($classField['type'])){
					$records[] = $classField['name'];
				}
				if(self::isDateField($classField['type'])){
					$dateFields[] = $classField['name'];
				}
			}
		}

		$val_nodes = [];
		if(file_exists($csvFile) && is_readable($csvFile)){
			switch(we_base_request::string('v', '', 'csv_enclosed')){
				case 'double_quote':
					$encl = '"';
					break;
				case 'single_quote':
					$encl = "'";
					break;
				default:
				case 'none':
					$encl = '';
					break;
			}

			$cp = (new we_import_CSV())
				->setData($data)
				->setDelim(we_base_request::_(we_base_request::RAW_CHECKED, 'v', '', 'csv_seperator'))
				->setEnclosure($encl);
			$cp->parseCSV();
			$recs = $cp->CSVFetchFieldNames();

			foreach($recs as $i => $cur){
				if(we_base_request::bool('v', false, 'csv_fieldnames') && $cur != ''){
					$val_nodes[$cur] = $cur;
				} else {
					$val_nodes['f_' . $i] = g_l('import', '[record_field]') . ($i + 1);
				}
			}
		}

		$th = [g_l('import', '[we_flds]'), g_l('import', '[rcd_flds]')];
		$rows = [];

		$i = 0;
		foreach($records as $record){
			$hdns .= we_html_element::hidden('records[' . $i . ']', $record);
			$sct_we_fields = (new we_html_select(['name' => 'we_flds[' . $record . ']',
					'class' => "weSelect",
					'onclick' => '',
					'style' => ''
					]))
				->addOption('', g_l('import', '[any]'));

			foreach($val_nodes as $value => $text){
				'@phan-var string $value';
				$b64_value = (string) (we_base_request::bool('v', false, 'startCSV') ? $value : base64_encode($value));
				$sct_we_fields->addOption($b64_value, htmlspecialchars((string) $text));
				if(isset($we_flds[$record])){
					if($value == base64_decode($we_flds[$record])){
						$sct_we_fields->selectOption($b64_value);
					}
				} elseif($value == $record){
					$sct_we_fields->selectOption($b64_value);
				}
			}

			switch($record){
				case 'Title':
					$new_record = g_l('import', '[we_title]');
					break;
				case 'Description':
					$new_record = g_l('import', '[we_description]');
					break;
				case 'Keywords':
					$new_record = g_l('import', '[we_keywords]');
					break;
				default:
					$new_record = '';
			}
			$rows[] = [
				($new_record != '') ? $new_record : $record,
				$sct_we_fields->getHTML()
			];
			++$i;
		}

		// Associated prefix selector.
		$asocPfx = (new we_html_table(['class' => 'default'], 1, 1))
			->setCol(0, 0, g_l('import', '[pfx]') . we_html_element::br(2) .
			we_html_tools::textInput('v[asoc_prefix]', ($v['asoc_prefix'] ?? g_l('import', ($v['import_type'] === self::TYPE_DOCUMENT ? '[pfx_doc]' : '[pfx_obj]'))), 255, [
				'onclick' => "WE().util.dialogTop(window).setFormField('v[rdo_filename]', true, 'radio', 0);"], 'text', '150px'));

		// Assigned record or attribute field selectors.
		$rcdPfxSelect = (new we_html_select(['name' => "v[rcd_pfx]",
				'class' => "weSelect",
				'onclick' => "WE().util.dialogTop(window).setFormField('v[pfx_fn]', 1, 'hidden'); WE().util.dialogTop(window).setFormField('v[rdo_filename]', true, 'radio', 1);",
				'style' => "width: 150px"
				]))
			->addOptions($val_nodes)
			->selectOption(we_base_request::string('v', '', 'rcd_pfx'));

		$asgndFld = (new we_html_table(['class' => 'default'], 1, 1))
			->setCol(0, 0, g_l('import', '[rcd_fld]') . we_html_element::br(2) . $rcdPfxSelect->getHTML());

		// Filename selector.
		$fn = (new we_html_table(['class' => 'default'], 5, 1))
			->setCol(0, 0, we_html_forms::radiobutton('0', (!isset($v['rdo_filename']) ? true : ($v['rdo_filename'] == 0)), 'v[rdo_filename]', g_l('import', '[auto]'), true, '', "self.document.we_form.elements['v[pfx_fn]'].value=0;"), [
				'colspan' => '2'])
			->setCol(1, 0, $asocPfx->getHTML(), ['style' => 'padding-left:25px;'])
			->setColAttributes(2, 0, ['height' => 5])
			->setCol(3, 0, we_html_forms::radiobutton('1', (!isset($v['rdo_filename']) ? false : ($v['rdo_filename'] == 1)), "v[rdo_filename]", g_l('import', '[asgnd]'), true, '', "self.document.we_form.elements['v[pfx_fn]'].value=1;"), [
				'colspan' => '2'])
			->setCol(4, 0, $asgndFld->getHTML(), ['style' => 'padding-left:25px;']);

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', '<br/>' . we_html_tools::dialogBorder(510, $rows, $th, '')));


		if(!empty($dateFields)){
			// Timestamp
			$tStamp = (new we_html_table(['class' => 'default withSpace'], 4, 1))
				->setCol(0, 0, we_html_forms::radiobuttonV('Unix', $v['rdo_timestamp'], "v[rdo_timestamp]", g_l('import', '[uts]'), true, '', '', false, g_l('import', '[unix_timestamp]'), 0, 384), [
					'colspan' => '2'])
				->setCol(1, 0, we_html_forms::radiobuttonV('GMT', $v['rdo_timestamp'], "v[rdo_timestamp]", g_l('import', '[gts]'), true, '', '', false, g_l('import', '[gmt_timestamp]'), 0, 384), [
					'colspan' => '2'])
				->setCol(2, 0, we_html_forms::radiobuttonV('Format', $v['rdo_timestamp'], "v[rdo_timestamp]", g_l('import', '[fts]'), true, '', '', false, g_l('import', '[format_timestamp]'), 0, 384), [
					'colspan' => '2'])
				->setCol(3, 0, we_html_tools::textInput("v[timestamp]", ($v['timestamp'] ?? ''), 0, ['onclick' => "WE().util.dialogTop(window).setFormField('v[rdo_timestamp]', true, 'radio', 2);"], 'text', '150px'), [
				'style' => 'padding-left:25px;']);

			$parts->add((new we_html_multiIconBoxContentLine(g_l('import', '[format_date]'), $tStamp->getHTML()))
					->space(we_html_multiIconBox::SPACE_MED2)
			);
			if(!isset($v['dateFields'])){
				$hdns .= we_html_element::hidden('v[dateFields]', implode(',', $dateFields));
			}
		}

		$conflict = $v['collision'] ?? we_exim_ExIm::COLLISION_RENAME;
		$fn_colsn = (new we_html_table(['class' => 'default withSpace'], 3, 1))
			->setCol(0, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_RENAME, $conflict, 'nameconflict', g_l('import', '[rename]'), true, 'defaultfont', "self.document.we_form.elements['v[collision]'].value='" . we_exim_ExIm::COLLISION_RENAME . "';"))
			->setCol(1, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_REPLACE, $conflict, 'nameconflict', g_l('import', '[replace]'), true, 'defaultfont', "self.document.we_form.elements['v[collision]'].value='" . we_exim_ExIm::COLLISION_REPLACE . "';"))
			->setCol(2, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_SKIP, $conflict, 'nameconflict', g_l('import', '[skip]'), true, 'defaultfont', "self.document.we_form.elements['v[collision]'].value='" . we_exim_ExIm::COLLISION_SKIP . "';"));

		$parts
			->add((new we_html_multiIconBoxContentLine(g_l('import', '[name_collision]'), $fn_colsn->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED2)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('import', '[name]'), $fn->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED2)
		);

		$znr = -1;

		return $hdns .
			(new we_html_multiIconBox($parts, 'csv'))
			->marginLeft()
			->fold($znr, false, g_l('weClass', '[moreProps]'), g_l('weClass', '[lessProps]'))
			->headline(g_l('import', '[assign_record_fields]'))
			->get();
	}

	protected function getWizCmdProgressStep(we_base_jsCmd $jsCmd, array $v, array $records, array $we_flds, array $attributes): string{
		$fields = [];
		switch($v['csv_enclosed']){
			case 'double_quote':
				$encl = '"';
				break;
			case 'single_quote':
				$encl = "'";
				break;
			default:
			case 'none':
				$encl = '';
				break;
		}
		[$v['classID']] = explode('_', $v['classID']);
		$csvFile = $v['uniquePath'] . '/temp_' . $v['cid'] . '.csv';
		if($v['encoding'] !== 'UTF-8'){
			we_base_file::save($csvFile, mb_convert_encoding(we_base_file::load($csvFile), 'UTF-8', $v['encoding'] . ',UTF-8') ?: '');
		}
		$cp = new we_import_CSV();
		$cp->setFile($csvFile);
		$cp->setDelim($v['csv_seperator']);
		$cp->setEnclosure($encl);
		$cp->parseCSV();
		$recs = [];
		$names = [];
		for($i = 0; $i < $cp->CSVNumFields(); $i++){
			$names[$i] = $cp->CSVFieldName($i);
			$recs[$names[$i]] = $cp->Fields[0][$i];
		}
		foreach($we_flds as $name => $value){
			$fields[$name] = ($recs[$value] ?? '');
		}
		if($v['pfx_fn'] == 1){
			$v['rcd_pfx'] = $recs[$v['rcd_pfx']];

			if($v['rcd_pfx'] == ''){
				$v['rcd_pfx'] = g_l('import', ($v['import_type'] === self::TYPE_DOCUMENT ? '[pfx_doc]' : '[pfx_obj]'));
			}
		}
		return parent::getWizCmdProgressStepBase($jsCmd, $v, $this->getHdns('v', $v) . $this->getHdns('records', $records) . $this->getHdns('we_flds', $we_flds), $fields
		);
	}

	protected function getWizCmdPrepareStep(we_base_jsCmd $jsCmd, array $v, array $records, array $we_flds, array $attrs, array $attributes): string{
		$encl = '';
		switch($v['csv_enclosed']){
			case 'double_quote':
				$encl = '"';
				break;
			case 'single_quote':
				$encl = "'";
				break;
			case 'none':
				$encl = '';
				break;
		}
		$cp = new we_import_CSV();
		$cp->setFile(WE_DOCUMENT_ROOT . $v['import_from']);
		$del = ($v['csv_seperator'] != '\t') ? (($v['csv_seperator'] != '') ? $v['csv_seperator'] : ' ') : '	';
		$cp->setDelim($del);
		$cp->setEnclosure($encl);
		$cp->parseCSV();
		$num_files = 0;
		$unique_id = we_base_file::getUniqueId();

		$path = TEMP_PATH . $unique_id;
		we_base_file::createLocalFolderByPath($path);
		we_base_file::insertIntoCleanUp($path, 3600);

		if($cp->isOK()){
			$fieldnames = ($v['csv_fieldnames']) ? 0 : 1;
			$num_rows = $cp->CSVNumRows();
			$num_fields = $cp->CSVNumFields();

			for($i = 0; $i < $num_rows + $fieldnames; $i++){
				$d = ['', ''];
				for($j = 0; $j < $num_fields; $j++){
					$d[1] .= (!$fieldnames ?
						(($cp->CSVFieldName($j) != '') ?
						$encl . str_replace($encl, '\\' . $encl, $cp->CSVFieldName($j)) . $encl :
						'') :
						$encl . 'f_' . $j . $encl);
					$d[0] .= ($fieldnames && $i == 0) ?
						(($cp->CSVFieldName($j) != '') ? $encl . str_replace($encl, '\\' . $encl, $cp->CSVFieldName($j)) . $encl : '') :
						(($cp->Fields[(!$fieldnames) ? $i : ($i - 1)][$j] != '') ?
						$encl . str_replace($encl, '\\' . $encl, (string) $cp->Fields[(!$fieldnames) ? $i : ($i - 1)][$j]) . $encl : '');
					if($j + 1 < $num_fields){
						$d[1] .= $del;
						$d[0] .= $del;
					}
				}
				we_base_file::save($path . '/temp_' . $i . '.csv', implode("\n", $d), 'wb');
				we_base_file::insertIntoCleanUp($path . '/temp_' . $i . '.csv');
				$num_files++;
			}
		}
		return parent::getWizCmdPrepareStepBase($jsCmd, $this->getHdns('v', $v) .
				$this->getHdns('records', $records) .
				$this->getHdns('we_flds', $we_flds) .
				we_html_element::hiddens([
					'v[numFiles]' => (string) $num_files,
					'v[uniquePath]' => $path
		]));
	}

}
