<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f642edb298bc $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-27 12:58 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class
 * @desc    class for tag <we:listview>
 *
 */
class we_listview_category extends we_listview_base{
	/** @var int */
	var $parentID = 0;

	/** @var int[] */
	var $catID = [];

	/** @var bool */
	var $hidedirindex = false;

	/**
	 * constructor of class
	 *
	 * @param   string $name          name of listview
	 * @param   int $rows          number of rows to display per page
	 * @param   int $offset        start offset of first page
	 * @param   string $order         field name(s) to order by
	 * @param   bool $desc          set to true, if order should be descendend
	 * @param   int $parentID         Id from Parent Category
	 * @param   int $cols   		  to display a table this is the number of cols
	 *
	 */
	function __construct(string $name, int $rows, int $offset, string $order, bool $desc, int $parentID, array $catID, int $cols, string $parentidname, bool $hidedirindex){
		parent::__construct($name, $rows, $offset, $order, $desc, [], false, [], $cols);
		$this->catID = $catID;
		$this->hidedirindex = $hidedirindex;

		$this->parentID = we_base_request::int($parentidname, $parentID);

		if(stripos($this->order, ' desc') !== false){//was #3849
			$this->order = str_ireplace(' desc', '', $this->order);
			$this->desc = true;
		}

		$this->order = trim($this->order);

		$orderstring = $this->order ? (' ORDER BY ' . $this->order . ($this->desc ? ' DESC' : '')) : '';

		$tail = ($this->catID ?
			'ID IN(' . implode(',', $this->catID) . ')' :
			'ParentID=' . $this->parentID
			);

		$this->DB_WE->query('SELECT
	ID,
	Text,
	Text AS Category,
	Path,
	Title,
	Description,
	Language' .
			($this->order === self::RANDOM_FIELD ? ', RAND() AS RANDOM' : '') . '
FROM ' .
			CATEGORY_TABLE . '
WHERE ' .
			$tail . ' ' .
			($this->order === self::RANDOM_FIELD ? 'ORDER BY RANDOM' : $orderstring) .
			($this->maxItemsPerPage ? (' LIMIT ' . $this->start . ',' . $this->maxItemsPerPage) : '')
		);
		$this->anz = $this->DB_WE->num_rows();
		$this->anz_all = $this->maxItemsPerPage ? fI('SELECT COUNT(1) FROM ' . CATEGORY_TABLE . ' WHERE ' . $tail) : $this->anz;

		$this->count = 0;
	}

	public function next_record(): bool{
		if($this->DB_WE->next_record()){
			$this->Record = array_change_key_case($this->DB_WE->getRecord());
			$this->count++;
			return true;
		}
		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}
		return false;
	}

}
