<?php
declare(strict_types=1);

/**
 * Zend Framework
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://framework.zend.com/license/new-bsd
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@zend.com so we can send you a copy immediately.
 *
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

/**
 * Performs LOGIN authentication
 *
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */
class we_mail_ProtocolSmtpLogin extends we_mail_ProtocolSmtp{
	/**
	 * LOGIN username
	 *
	 * @var string
	 */
	protected $_username;

	/**
	 * LOGIN password
	 *
	 * @var string
	 */
	protected $_password;

	/**
	 * Constructor.
	 *
	 * @param  string $host   (Default: 127.0.0.1)
	 * @param  int    $port   (Default: null)
	 * @param  array  $config Auth-specific parameters
	 * @return void
	 */
	public function __construct(string $host = '127.0.0.1', int $port = 0, array $config = []){
		if(isset($config['username'])){
			$this->_username = $config['username'];
		}
		if(isset($config['password'])){
			$this->_password = $config['password'];
		}

		parent::__construct($host, $port, $config);
	}

	/**
	 * Perform LOGIN authentication with supplied credentials
	 *
	 * @return void
	 */
	public function auth(): void{
		// Ensure AUTH has not already been initiated.
		parent::auth();

		$this->_send('AUTH LOGIN');
		$this->_expect('334');
		$this->_send(base64_encode($this->_username));
		$this->_expect('334');
		$this->_send(base64_encode($this->_password));
		$this->_expect('235');
		$this->_auth = true;
	}

}
