<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e4301c9e291b $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-13 12:33 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @abstract This class executes commands and provides views depending on requesting client.
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpc_cmdShell{
	/** @var string */
	protected $Protocol;

	/** @var we_rpc_cmd */
	protected $Mod;

	/** @var string */
	protected $ModName;

	/** @var string */
	protected $Cmd;

	/** @var we_rpc_genericJSONView */
	protected $View;

	/** @var we_rpc_response */
	protected $Response;

	/** @var int */
	protected $Status = we_rpc_cmd::STATUS_OK;

	public function __construct(string $protocol, string $mod, string $cmd){
		$this->Protocol = $protocol;
		$this->ModName = $mod;
		$this->Mod = $this->createCmd();
		$this->Cmd = $cmd;

		if(($view = we_base_request::string('view'))){
			if(!$this->isViewAllowed($view)){
				$this->Status = we_rpc_cmd::STATUS_NOT_ALLOWED_VIEW;
			}
		}/* else {
		  $view = $this->ModName;
		  } */
		if($this->Status == we_rpc_cmd::STATUS_OK){
			$this->View = $this->getView();
		}
	}

	private function createCmd(): we_rpc_cmd{
		$classname = 'we_rpcCmd_' . $this->ModName;
		if(!class_exists($classname, true)){
			$this->Status = we_rpc_cmd::STATUS_NO_CMD;
			t_e('RPC command not found', $this->ModName, $_REQUEST);
			throw new InvalidArgumentException('RPC command not found:' . $this->ModName);
		}

		$obj = new $classname($this);
		$obj->executeRpcCmd($this);
		$this->Status = $obj->getStatus();

		return $obj;
	}

	function getView(): we_rpc_genericJSONView{
		return new we_rpc_genericJSONView($this->Protocol);
	}

	private function isViewAllowed(string $view): bool{
		return ($view === $this->ModName);
	}

	function executeCommand(): void{
		$this->Response = $this->Mod->execute();
	}

	function cleanUp(): void{
		$this->Mod->cleanUp();
	}

	function getResponse(): string{
		return $this->View->getResponse($this->Response);
	}

	function executeInternalCmd(&$cmd): we_rpc_response{
		$cmd = $this->createCmd();
		return $cmd->execute();
	}

	function getInternalView(string $cmd): string{
		$View = $this->getView();
		return $View->getResponse($this->executeInternalCmd($cmd));
	}

	public function getErrorOut(): string{
		switch($this->Status){
			case we_rpc_cmd::STATUS_NO_CMD:
				return 'ERROR: No command defined!';
			case we_rpc_cmd::STATUS_NO_VIEW:
				return 'ERROR: No view defined!';
			case we_rpc_cmd::STATUS_NO_SESSION:
				return 'ERROR: No session exists!';
			default:
				return 'ERROR';
		}
	}

	function getStatus(): int{
		return $this->Status;
	}

	public function getCmd(): string{
		return $this->Cmd;
	}

}
