<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 05797a860f4b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-21 15:36 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_schedule_entry extends we_schedule_scheduler{//fixme splitup in specific entries
	/** @var bool */
	public $active = false;

	/** @var string */
	public $rerun = self::RERUN_ONCE;

	/** @var string */
	public $task = self::SCHEDULE_FROM;

	/** @var int */
	public $parentID = 0;

	/** @var int */
	public $doctypeID = 0;

	/** @var bool */
	public $doctypeDefault = false;

	/** @var int[] */
	public $categoryIDs = [];

	/** @var DateTime */
	public $time = null;

	/** @var int[] */
	public $months = [];

	/** @var int[] */
	public $days = [];

	/** @var int[] */
	public $weekdays = [];

	/**
	 * @param bool $active
	 * @param string $rerun
	 * @param string $task
	 * @param int $parentID
	 * @param int $doctypeID
	 * @param bool $doctypeDefault
	 * @param array $categoryIDs
	 * @param DateTime $time
	 * @param int[] $months
	 * @param int[] $days
	 * @param int[] $weekdays
	 */
	protected function __construct(bool $active, string $rerun, string $task, int $parentID, int $doctypeID, bool $doctypeDefault, array $categoryIDs, DateTime $time, array $months, array $days, array $weekdays){
		$this->active = $active;
		$this->rerun = $rerun;
		$this->task = $task;
		$this->parentID = $parentID;
		$this->doctypeID = $doctypeID;
		$this->doctypeDefault = $doctypeDefault;
		$this->categoryIDs = $categoryIDs;
		$this->time = $time;
		$this->months = $months;
		$this->days = $days;
		$this->weekdays = $weekdays;
	}

	public static function emptyEntry(): self{
		return new self(true, self::RERUN_ONCE, self::SCHEDULE_FROM, 0, 0, false, [], new DateTime(), array_fill(0, 12, 0), array_fill(0, 31, 0), array_fill(0, 7, 0));
	}

	public function isSchedulerActive(): bool{
		return $this->task === self::SCHEDULE_FROM && $this->active;
	}

	public function addCat(int $id): void{
		if(in_array($id, $this->categoryIDs)){
			return;
		}
		$this->categoryIDs[] = $id;
	}

	public function delCat(int $id): void{
		if(($pos = array_search($id, $this->categoryIDs, false)) === false){
			return;
		}
		unset($this->categoryIDs[$pos]);
	}

	public function clearCats(): void{
		$this->categoryIDs = [];
	}

	public function getNextTimestamp(): DateTime{
		static $now = null;
		if(!$now){
			$now = new DateTime();
		}
		$nextTime = clone $this->time;
		switch($this->rerun){
			default:
			case self::RERUN_ONCE:
				return $nextTime;
			case self::RERUN_HOUR:
				while($nextTime < $now){
					$nextTime->add(new DateInterval('PT1H'));
				}
				return $nextTime;
			case self::RERUN_DAY:
				while($nextTime < $now){
					$nextTime->add(new DateInterval('P1D'));
				}
				return $nextTime;
			case self::RERUN_WEEK:
				$wdayNow = (int) $now->format('w');
				$nextTime = clone $now;
				$nextTime->setTime((int) $this->time->format('H'), (int) $this->time->format('i'));
				if($this->weekdays[$wdayNow] && ($nextTime > $now)){ // wenn am heutigen Tag was geschehen soll, checken ob Ereignis noch offen, wenn ja dann speichern
					return $nextTime;
				}
				$found = false;
				// naechst moeglicher Wochentag suchen
				for($wd = $wdayNow + 1; $wd <= 6; $wd++){
					$nextTime->add(new DateInterval('P1D'));
					if($this->weekdays[$wd]){
						$found = true;
						break;
					}
				}
				if(!$found){
					for($wd = 0; $wd <= $wdayNow; $wd++){
						$nextTime->add(new DateInterval('P1D'));
						if($this->weekdays[$wd]){
							$found = true;
							break;
						}
					}
				}
				if(!$found){
					throw new we_exception_base('next time not found');
				}
				return $nextTime;

			case self::RERUN_MONTH:
				$dayNow = (int) $now->format('j');
				$nextTime = clone $now;
				$nextTime->setTime((int) $this->time->format('H'), (int) $this->time->format('i')); // zeit fuer heutigen tag
				if($this->days[$dayNow - 1] && ($nextTime > $now)){ // wenn am heutigen Tag was geschehen soll, checken ob Ereignis noch offen, wenn ja dann speichern
					return $nextTime;
				}

				$nextTime->add(new DateInterval('P1D'));
				$dayTomorrow = (int) $nextTime->format('j');

				$trys = 0;
				while(!$this->days[$dayTomorrow - 1] && $trys <= 365){
					$nextTime->add(new DateInterval('P1D'));
					$dayTomorrow = (int) $nextTime->format('j');
					$trys++;
				}
				if($trys <= 365){
					return $nextTime;
				}
				throw new we_exception_base('date not valid');
			case self::RERUN_YEAR:
				$dayNow = (int) $now->format('j');
				$monthNow = (int) $now->format('m');
				$nextTime = clone $now;
				$nextTime->setTime((int) $this->time->format('H'), (int) $this->time->format('i')); // zeit fuer heutigen tag
				if($this->days[$dayNow - 1] && $this->months[$monthNow - 1] && ($nextTime > $now)){ // wenn am heutigen Tag was geschehen soll, checken ob Ereignis noch offen, wenn ja dann speichern
					return $nextTime;
				}

				$nextTime->add(new DateInterval('P1D'));
				$dayTomorrow = (int) $nextTime->format('j');
				$monthTomorrow = (int) $nextTime->format('m');

				$trys = 0;
				while((!$this->days[$dayTomorrow - 1] || !$this->months[$monthTomorrow - 1] ) && $trys <= 365){
					$nextTime->add(new DateInterval('P1D'));
					$dayTomorrow = (int) $nextTime->format('j');
					$monthTomorrow = (int) $nextTime->format('m');
					$trys++;
				}
				if($trys <= 365){
					return $nextTime;
				}
				throw new we_exception_base('date not valid');
		}
	}

	public function getSaveData(int $id, string $table): array{
		$Wann = $this->getNextTimestamp();
		return [
			'DID' => $id,
			'documentTable' => stripTblPrefix($table),
			'expire' => $Wann->format(we_database_base::DATE_TIME_FORMAT),
			'task' => $this->task,
			'rerun' => $this->rerun,
			'Active' => $this->active,
			'months' => $this->rerun === self::RERUN_YEAR ? sql_function('b\'' . implode('', $this->months) . '\'') : null,
			'days' => in_array($this->rerun, [self::RERUN_YEAR, self::RERUN_MONTH]) ? sql_function('b\'' . implode('', $this->days) . '\'') : null,
			'weekdays' => $this->rerun === self::RERUN_WEEK ? sql_function('b\'' . implode('', $this->weekdays) . '\'') : null,
			'time' => $this->time->format(we_database_base::DATE_TIME_FORMAT),
			'ParentID' => $this->parentID,
			'DoctypeID' => $this->doctypeID,
			'DoctypeDefault' => $this->doctypeDefault,
			'CategoryIDs' => $this->categoryIDs ?: null,
		];
	}

	/**
	 *
	 * @param array{Active:int,rerun:string,task:string,ParentID:int,DoctypeID:int,DoctypeDefault:int,CategoryIDs:string,time:string,months:string,days:string,weekdays:string} $hash
	 * @return \self
	 */
	public static function fromHash(array $hash): self{
		return new self((bool) $hash['Active'], $hash['rerun'], $hash['task'], (int) $hash['ParentID'], (int) $hash['DoctypeID'], (bool) $hash['DoctypeDefault'], $hash['CategoryIDs'] === null ? [
			] : we_base_request::intList($hash['CategoryIDs']), new DateTime($hash['time']), $hash['months'] === null ? array_fill(0, 12, 0) : array_map('intval', str_split((string) $hash['months'])), $hash['days'] === null ? array_fill(0, 31, 0) : array_map('intval', str_split((string) $hash['days'])), $hash['weekdays'] === null ? array_fill(0, 7, 0) : array_map('intval', str_split((string) $hash['weekdays'])));
	}

	public static function fromRequest(int $pos): self{
		$timeR = we_base_request::_(we_base_request::STRING, 'we', we_base_request::TYPE_DEFAULT, 'schedule', $pos, self::SUBMIT_TIME);
		$time = new DateTime(($timeR['date'] ?? '1970-01-01') . ' ' . ($timeR['time'] ?? '00:' . ($timeR['min'] ?? '00') . ':00'));
		$ret = [];
		foreach(self::persistent_slots as $name => $type){
			$ret[$name] = we_base_request::_($type, 'we', we_base_request::TYPE_DEFAULT, 'schedule', $pos, $name);
		}
		return new we_schedule_entry($ret[self::SUBMIT_ACTIVE], $ret[self::SUBMIT_RERUN], $ret[self::SUBMIT_TASK], $ret[self::SUBMIT_PARENT], $ret[self::SUBMIT_DOCTYPE], $ret[self::SUBMIT_DOCTYPE_DEFAULT], $ret[self::SUBMIT_CATEGORY], $time, $ret[self::SUBMIT_MONTHS], $ret[self::SUBMIT_DAYS], $ret[self::SUBMIT_WDAYS]);
	}

}
