<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2ec51e6b74e5 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-03-03 12:51 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_shop_articleExtend{
	private static $typeObj;
	private static $typeDoc;
	private static $orderBy;
	private static $classid;
	private static $actPage;

	/*	 * ************ fuction for orders  ************** */

	private static function getTitleLinkObj(string $text, string $orderKey): string{
		$href = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_article_extend' .
			'&typ=' . self::$typeObj .
			'&orderBy=' . $orderKey .
			'&ViewClass=' . self::$classid .
			'&actPage=' . self::$actPage .
			( (self::$orderBy == $orderKey && !we_base_request::bool('orderDesc')) ? '&orderDesc=1' : '' );

		return '<a href="' . $href . '">' . $text . '</a>' . (self::$orderBy == $orderKey ? ' <i class="fa fa-sort-' . (we_base_request::bool('orderDesc') ? 'desc' : 'asc') . ' fa-lg"></i>' : '<i class="fa fa-sort fa-lg"></i>');
	}

	private static function getPagerLinkObj(): string{
		return WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_article_extend' .
			'&typ=' . self::$typeObj .
			'&orderBy=' . self::$orderBy .
			'&ViewClass=' . self::$classid .
			'&actPage=' . self::$actPage .
			(we_base_request::bool('orderdesc') ? '&orderDesc=1' : '' );
	}

	private static function getTitleLinkDoc(string $text, string $orderKey): string{
		$href = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_article_extend' .
			'&typ=' . self::$typeDoc .
			'&orderBy=' . $orderKey .
			'&actPage=' . self::$actPage .
			( (self::$orderBy == $orderKey && !we_base_request::bool('orderDesc')) ? '&orderDesc=1' : '' );

		$arrow = '<i class="fa fa-sort fa-lg"></i>';

		if(self::$orderBy == $orderKey){
			$arrow = ' <i class="fa fa-lg fa-sort-' . (we_base_request::bool('orderDesc') ?
				'desc' :
				'asc') .
				'"></i>';
		}

		return '<a href="' . $href . '">' . $text . '</a>' . $arrow;
	}

	private static function getPagerLinkDoc(): string{
		return WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_article_extend' .
			'&typ=' . self::$typeDoc .
			'&orderBy=' . self::$orderBy .
			'&actPage=' . self::$actPage .
			(we_base_request::bool('orderdesc') ? '&orderDesc=1' : '' );
	}

	private static function array_select(string $select_name, string $label): string{ // function for a selectbox for the purpose of selecting a class
		$shopConfig = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));

		$fe = (isset($shopConfig[3]) ?
			array_map('intval', array_filter(explode(',', $shopConfig[3]))) : //determine more than just one class-ID
			[]);

		$selVal = we_base_request::string($select_name);

		$menu = '<label for="' . $select_name . '">' . $label . '</label><select name="' . $select_name . '" onchange="document.location.href=\'' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_article_extend&typ=object&ViewClass=\' + this.options[this.selectedIndex].value ">';

		if($fe){
			foreach(DB_WE::$inst->getAllFirstq('SELECT ID,Text FROM ' . OBJECT_TABLE . ' WHERE ID IN (' . implode(',', $fe) . ')', false) as $id => $val){
				$menu .= '<option value="' . $id . '"' . (($id == $selVal) ? ' selected="selected"' : '') . '>' . $val . '</option>';
			}
		}
		$menu .= '</select><input type="hidden" name="typ" value="object" />';

		return $menu;
	}

	public static function getDialog(): string{
		self::$typeObj = we_base_request::string('type', 'object');
		self::$typeDoc = we_base_request::string('type', 'document');
		$DB_WE = DB_WE::$inst;
		self::$actPage = we_base_request::int('actPage', 0);

		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		/* $waehr = '&nbsp;' . htmlspecialchars($feldnamen[0]);
		  $numberformat = $feldnamen[2];
		 */
//determine more than just one class-ID
		$fe = !empty($feldnamen[3]) ? explode(',', $feldnamen[3]) : [0];

		self::$classid = intval($fe[0]);
		$nrOfPage = (!empty($feldnamen[4]) && $feldnamen[4] !== 'default') ? (int) $feldnamen[4] : 20;
		//$varies = "variant_" . we_base_variants::ELEMENT_NAME;

		/*		 * ************ some config  ************** */
		$parts = new we_html_multiIconBoxContent();

		if(!isset($_REQUEST['sort'])){
			$_REQUEST['sort'] = '';
		}
		$topInfo = '';

		switch(we_base_request::string('typ')){
			case 'object': //start output object
				self::$orderBy = $DB_WE->escape(we_base_request::string('orderBy', 'obTitle'));
				$entries = 0;
				$count_expression = $from_expression = $where_expression = [];
				if(empty($fe)){
					if(!self::$classid){
						break;
					}
					$count_expression[] = 'COUNT(obx.OF_ID)';
					$from_expression[] = OBJECT_X_TABLE . self::$classid . ' obx';
					$where_expression[] = 'obx.OF_ID!=0';
				} else {
					foreach($fe as $clId){
						$clId = intval($clId);
						if(!$clId){
							continue;
						}
						$count_expression[] = 'COUNT(DISTINCT ob' . $clId . '.OF_ID)';
						$from_expression[] = OBJECT_X_TABLE . $clId . ' ob' . $clId;
						$where_expression[] = 'ob' . $clId . '.OF_ID!=0';
					}
				}
				if($count_expression){
					$entries += array_sum($DB_WE->getAllq('SELECT ' . implode('+', $count_expression) . ' FROM ' . implode(',', $from_expression) . ' WHERE ' . implode(' AND ', $where_expression), true)); // Pager: determine the number of records;
				}
				//$active_page = we_base_request::_(we_base_request::RAW, 'page', 0); // Pager: determine the current page
				$docType2 = we_base_ContentTypes::OBJECT_FILE; // Pager: determine the current page
				//$typeAlias = we_base_ContentTypes::OBJECT; // Pager: determine the current page
				$classSelectTable = '';
				if($entries){ // Pager: Number of records not empty?
					$topInfo = ($entries > 0 ? $entries : g_l('modules_shop', '[noRecord]'));
					self::$classid = abs(we_base_request::int('ViewClass')); // gets the value from the selectbox;

					$classSelectTable .= '<table style="width:600px;">
    <tr>
        <td colspan="2" class="defaultfont">' .
						// displays a selectbox for the purpose of selecting a class..
						self::array_select('ViewClass', g_l('modules_shop', '[classSel]')) . '</td>
    </tr>
</table>';
					$parts->add(new we_html_multiIconBoxContentLine('', $classSelectTable));

					// :: then do the query for objects
					$orderRows = $DB_WE->getAllq('SELECT
	obx.input_' . we_shop_shop::TITLE_FIELD_NAME . ' AS obTitle,
	obx.OF_ID AS obID,
	DATE_FORMAT(of.Creation_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS cDate,
	DATE_FORMAT(of.Publish_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS cPub,
	DATE_FORMAT(of.Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS cMob,
	of.IsPublished
FROM ' .
						OBJECT_X_TABLE . self::$classid . ' obx JOIN ' .
						OBJECT_FILES_TABLE . ' `of` ON obx.OF_ID=of.ID
WHERE
	of.IsFolder=0
ORDER BY
	obx.OF_ID'); // get the shop-objects from DB;
					// build the table
					// we need functionalitty to order these
					/* if(we_base_request::bool('orderBy')){
					  usort($orderRows, 'orderBy');
					  } */

					// build the headline
					$headline = [
						self::getTitleLinkObj(g_l('modules_shop', '[ArtName]'), 'obTitle'),
						self::getTitleLinkObj(g_l('modules_shop', '[ArtID]'), 'obID'),
						self::getTitleLinkObj(g_l('modules_shop', '[artCreate]'), 'cDate'),
						self::getTitleLinkObj(g_l('modules_shop', '[artPub]'), 'cPub'),
						self::getTitleLinkObj(g_l('modules_shop', '[artMod]'), 'cMob'),
					];

					$content = [];

					for($i = (self::$actPage * $nrOfPage); $i < count($orderRows) && $i < (self::$actPage * $nrOfPage + $nrOfPage); $i++){
						$isPublished = (bool) $orderRows[$i]['IsPublished'];
						$publishedStylePre = $isPublished ? [] : ['style' => "color: red"];

						$content[] = [
							'<a href="javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_FILES_TABL,\'' . $orderRows[$i]['obID'] . '\',\'' . $docType2 . '\');");">' . we_html_element::span($publishedStylePre, substr($orderRows[$i]['obTitle'], 0, 25) . '..') . '</a>',
							we_html_element::span($publishedStylePre, $orderRows[$i]['obID']),
							we_html_element::span($publishedStylePre, $orderRows[$i]['cDate'] ?: ''),
							$orderRows[$i]['cPub'] ?: '',
							we_html_element::span($publishedStylePre, $orderRows[$i]['cMob'] ?: '')
						];
					}

					$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::dialogBorder(670, $content, $headline)))
							->noline())

						// now the pager class at last:
						// Pager: Zweite Linkliste zeigen
						->add(new we_html_multiIconBoxContentLine('', we_shop_pager::getStandardPagerHTML(self::getPagerLinkObj(), self::$actPage, $nrOfPage, count($orderRows))));
				}
				break;
			case 'document': //start output doc
				self::$orderBy = we_base_request::_(we_base_request::RAW, 'orderBy', 'sql');
				$entries = fI('SELECT COUNT(Name) FROM ' . CONTENT_TABLE . ' WHERE nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\''); // Pager: determine the number of records;
				//$active_page = we_base_request::_(we_base_request::RAW, 'page', 0); // Pager: determine the number of records;
				$docType = we_base_ContentTypes::WEDOCUMENT; // Pager: determine the current page
				//$typeAlias = "document"; // Pager: determine the current page

				if($entries){ // Pager: Number of records not empty?
					$topInfo = $entries;
					// :: then do the query for documents
					$orderRows = $DB_WE->getAllq('SELECT
	c.dat AS `sql`,
	c.DID AS dd,
	DATE_FORMAT(f.Creation_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS dDate,
	DATE_FORMAT(f.Publish_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS dPub,
	f.IsPublished,
	DATE_FORMAT(f.Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS dMod
FROM ' .
						CONTENT_TABLE . ' c JOIN ' .
						FILE_TABLE . ' f ON f.ID=c.DID
WHERE
	c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND
	c.nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\'
ORDER BY dd'); // get the shop-documents from DB;
					// for the articlelist, we need also all these article, so save them in array
					// we need functionalitty to order these
					/* if(we_base_request::bool('orderBy')){
					  usort($orderRows, 'orderBy');
					  } */
					//$typeAlias = "document";
					// build the headline
					$headline = [
						self::getTitleLinkDoc(g_l('modules_shop', '[ArtName]'), 'sql'),
						self::getTitleLinkDoc(g_l('modules_shop', '[ArtID]'), 'dd'),
						self::getTitleLinkDoc(g_l('modules_shop', '[artCreate]'), 'dDate'),
						self::getTitleLinkDoc(g_l('modules_shop', '[artPub]'), 'dPub'),
						self::getTitleLinkDoc(g_l('modules_shop', '[artMod]'), 'dMod'),
					];

					$content = [];
					for($nr = 0, $i = (self::$actPage * $nrOfPage); $i < count($orderRows) && $i < (self::$actPage * $nrOfPage + $nrOfPage); $i++, $nr++){
						$isPublished = (bool) $orderRows[$i]['IsPublished'];
						$publishedStylePre = $isPublished ? [] : ['style' => "color: red"];
						$content[$nr] = [
							'<a href="javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.FILE_TABLE,\'' . $orderRows[$i]['dd'] . '\',\'' . $docType . '\');");">' . we_html_element::span($publishedStylePre, substr($orderRows[$i]['sql'], 0, 25) . '..') . '</a>',
							we_html_element::span($publishedStylePre, $orderRows[$i]['dd']),
							we_html_element::span($publishedStylePre, $orderRows[$i]['dDate'] ?: ''),
							$orderRows[$i]['dPub'] ?: '',
							we_html_element::span($publishedStylePre, $orderRows[$i]['dMod'] ?: ''),
						];
					}
					$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::dialogBorder(670, $content, $headline)))
							->noline())
						->add(new we_html_multiIconBoxContentLine('', we_shop_pager::getStandardPagerHTML(self::getPagerLinkDoc(), self::$actPage, $nrOfPage, count($orderRows))));
				}

				/*				 * ******** END PROCESS THE OUTPUT IF OPTED FOR A DOCUMENT *********** */
				break;
			default:
				return 'Die von Ihnen gewünschte Seite kann nicht angezeigt werden!';
		}


		if($parts->isEmpty()){
			$parts->add(new we_html_multiIconBoxContentLine('', '<table style="width:100%">' .
					'<tr><td class="defaultfont">' . g_l('modules_shop', '[noRecordAlert]', false, true) . '</td></tr>' .
					'<tr><td class="defaultfont">' . we_html_button::create_button('fa:btn_shop_pref,fa-lg fa-pencil,fa-lg fa-list-alt', "javascript:top.opener.top.we_cmd('settings')", '', "", !we_base_permission::hasPerm([
							we_base_perm::USER_NEW])) . '</td></tr>' .
					'</table>'));
			$topInfo = g_l('modules_shop', '[noRecord]');
		}

		return we_html_tools::getHtmlTop('', '', '
<body class="weEditorBody" onload="self.focus();top.content.editor.edheader.weTabs.setFrameSize();" onresize="top.content.editor.edheader.weTabs.setFrameSize();" onunload="">
<form action="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_article_extend" method="post">' .
				(new we_html_multiIconBox($parts, 'revenues'))->marginLeft()->headline(sprintf(g_l('tabs', '[module][artList]'), $topInfo))->get() . '
		</form></body>
</html>');
	}

}
