<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2f1aeda46386 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-13 01:48 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_versions_search{
	/** @var we_database_base */
	private $db;

	/** @var string[] */
	public $searchFields = [];

	/** @var string[] */
	public $location = [];

	/** @var string[] */
	public $search = [];

	/** @var int */
	public $mode = 0;

	/** @var string */
	public $order = 'version DESC';

	/** @var int */
	public $anzahl = 25;

	/** @var int */
	public $searchstart = 0;

	/** @var int */
	public $height = 1;

	/** @var string */
	public $where;

	/** @var we_versions_view */
	public $View;

	/**
	 *  Constructor for class 'weVersionsSearch'
	 */
	public function __construct(we_versions_view $view = null){
		$this->db = new DB_WE();
		$this->View = $view ?: new we_versions_view();
	}

	/**
	 * @abstract make WHERE-Statement for mysql-SELECT
	 */
	public function getWhere(we_versions_model $model): string{
		$currentSearchFields = $model->getProperty('currentSearchFields');
		if(!$model->mode || empty($currentSearchFields)){
			return '';
		}
		$where = $modConst = [];
		$currentSearch = $model->getProperty('currentSearch');
		$currentLocation = $model->getProperty('currentLocation');

		foreach($currentSearchFields as $k => $v){
			switch($v){
				case 'modifierID':
					if(isset($currentSearch[$k])){
						$where[] = $v . '="' . DB_WE::$inst->escape($currentSearch[$k]) . '"';
					}
					break;
				case 'status':
					if(isset($currentSearch[$k])){
						$where[] = $v . '="' . DB_WE::$inst->escape($currentSearch[$k]) . '"';
					}
					break;
				case 'logDate':
					if(isset($currentLocation[$k]) && isset($currentSearch[$k]) && $currentSearch[$k]){
						$date = new DateTime($currentSearch[$k]);
						switch($currentLocation[$k]){
							case we_search_base::OP_IS:
								$where[] = $v . ' BETWEEN "' . $date->format(we_database_base::DATE_FORMAT) . '" AND ("' . $date->format(we_database_base::DATE_FORMAT) . '" + INTERVAL 1 DAY)';
								break;
							case we_search_base::OP_LO:
								$where[] = $v . '<"' . $date->format(we_database_base::DATE_FORMAT) . '"';
								break;
							case we_search_base::OP_LEQ:
								$where[] = $v . '<="' . $date->format(we_database_base::DATE_FORMAT) . '"';
								break;
							case we_search_base::OP_HI:
								$where[] = '>"' . $date->format(we_database_base::DATE_FORMAT) . '"';
								break;
							case we_search_base::OP_HEQ:
								$where[] = $v . '>="' . $date->format(we_database_base::DATE_FORMAT) . '"';
								break;
						}
					}
					break;
				case 'allModsIn':
					if(isset($currentSearch[$k]) && in_array($currentSearch[$k], we_versions_version::MOD_FIELDS)){
						$modConst[] = $currentSearch[$k];
					}
					break;
			}
		}
		if($modConst){
			//this searches for any modification matches the given ones.
			$ids = $this->db->getAllq('SELECT ID FROM ' . VERSIONS_TABLE . ' WHERE modifiedFields IS NOT NULL AND CONCAT(",",modifiedFields,",") REGEXP ",(' . implode('|', $modConst) . ')," ', true);
			$where[] = ($ids ?
				'ID IN (' . implode(',', $ids) . ') ' :
				'0');
		}

		return implode(' AND ', $where);
	}

	/**
	 * @abstract get modification-fields for filter-SELECT
	 * @return array<string,string> of modification-fields
	 */
	public static function getModFields(): array{
		$modFields = [];

		foreach(we_versions_version::MOD_FIELDS as $k){
			if($k != 'status'){
				$modFields[$k] = g_l('versions', '[' . $k . ']');
			}
		}

		return $modFields;
	}

	/**
	 * @abstract get filter categories for filter-SELECT
	 * @return array<string,string> of filter categories
	 */
	public function getFields(): array{
		return [
			'allModsIn' => g_l('versions', '[allModsIn]'),
			'logDate' => g_l('versions', '[modTime]'),
			'modifierID' => g_l('versions', '[modUser]'),
			'status' => g_l('versions', '[status]')
		];
	}

	/**
	 * @abstract get all user for filter-SELECT in category 'modifierID'
	 * @return array<int,string> of users
	 */
	public function getUsers(): array{
		//@phan-suppress-next-line PhanPartialTypeMismatchReturn
		return DB_WE::$inst->getAllFirstq('SELECT ID,username FROM ' . USER_TABLE . ' WHERE Type="user"' .
				(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? '' : ' AND ParentID=' . $_SESSION['user']['Group']) . ' ORDER BY username', false);
	}

	/**
	 * @abstract get status
	 * @return array<string,string> of stats
	 */
	public function getStats(): array{
		return [
			'published' => g_l('versions', '[published]'),
			'unpublished' => g_l('versions', '[unpublished]'),
			'saved' => g_l('versions', '[saved]'),
			'deleted' => g_l('versions', '[deleted]'),
		];
	}

}
