/* global WE, top */

/**
 * webEdition CMS
 *
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

/**CLASS:
 //   EditorFrameset
 //
 // DESCRIPTION:
 //   Manages access to one editor-window.
 //
 // ARGUMENTS:
 //   none
 //
 // RETURNS:
 //   none
 */
WE().layout.EditorFrame = function (ref, elementId) {
	this.FrameId = elementId;
	this.EditorFrameWindow = top.window.frames[elementId]; //bm_content_frame
	this.EditorFrameReference = ref; // not needed yet !
	this.EditorType = null;	// model|cockpit|none_webedition, etc
	this.EditorTransaction = null; // is set
	this.EditorDocumentId = 0; // is set
	this.EditorEditorTable = ""; // is set
	this.EditorIsLoading = true; // is set
	this.EditorIsHot = false;
	this.EditorEditPageNr = null;

	// seeMode - url, parameters
	this.EditorUrl = "";
	this.EditorDocumentParameters = "";

	this.EditorEditCmd = "";

	// checkboxes in Editor-Footer:
	this.EditorMakeNewDoc = false;
	this.EditorPublishWhenSave = false;
	this.EditorAutoRebuild = false;
	this.EditorMakeSameDoc = false;

	// wysiwyg in editors
	this.EditorDidSetHiddenText = false;

	// information for tabs
	this.EditorDocumentText = " &hellip; ";
	this.EditorDocumentPath = null;
	this.EditorContentType = ""; // is set

	this.EditorIsActive = false;
	this.EditorIsInUse = false;

	// reload needed?
	this.EditorReloadNeeded = false;
	this.EditorReloadAllNeeded = false;
};

WE().layout.EditorFrame.prototype = {
	/*functionality*/

	freeEditor: function () {
		this.setEmptyEditor();
		this.EditorType = null;	// model|cockpit, etc
		this.EditorTransaction = null;
		this.EditorDocumentId = 0;
		this.EditorEditorTable = "";
		this.EditorIsLoading = true;
		this.EditorIsHot = false;
		this.EditorUrl = "";
		this.EditorDocumentParameters = "";
		this.EditorEditCmd = "";
		// checkboxes in Editor-Footer:
		this.EditorMakeNewDoc = false;
		this.EditorPublishWhenSave = false;
		this.EditorAutoRebuild = false;
		this.EditorMakeSameDoc = false;

		// wysiwyg in editors
		this.EditorDidSetHiddenText = false;

		// information for tabs
		this.EditorDocumentText = " &hellip; ";
		this.EditorDocumentPath = null;
		this.EditorContentType = "";
		this.EditorEditPageNr = null;
		this.EditorIsActive = false;
		this.EditorIsInUse = false;
	},
	initEditorFrameData: function (obj) {
		this.EditorIsInUse = true;
		if (!obj) {
			return;
		}
		for (var eigen in obj) {
			this[eigen] = obj[eigen];
		}
		this.updateEditorTab();
	},
	updateEditorTab: function () {
		this.EditorDocumentText = this.EditorDocumentText.replace(/</g, "&lt;").replace(/>/g, "&gt;");
		WE().layout.multiTabs.setText(this.FrameId, this.EditorDocumentText);
		WE().layout.multiTabs.setTitle(this.FrameId, this.EditorDocumentPath + (this.EditorType === "model" ? " (ID: " + this.EditorDocumentId + ")" : ""));
		WE().layout.multiTabs.setId(this.FrameId, (this.EditorType === "model" ? "ID: " + this.EditorDocumentId : this.EditorDocumentText));
		WE().layout.multiTabs.setContentType(this.FrameId, this.EditorContentType);
		WE().layout.multiTabs.setLoading(this.FrameId, this.EditorIsLoading);
		WE().layout.multiTabs.setModified(this.FrameId, this.EditorIsHot);
	},
	/**
	 // FUNCTION:
	 //   setEmptyEditor
	 //
	 // DESCRIPTION:
	 //   if all editors are closed, the editor shows an emtpy page
	 //
	 // ARGUMENTS:
	 //   none
	 //
	 // RETURNS:
	 //   nothing
	 */
	setEmptyEditor: function () {
		this.EditorFrameWindow.location = "about:blank";
	},
	getEditorFrameWindow: function () {
		return this.EditorFrameWindow;
	},
	getDocumentReference: function () {
		return this.getEditorFrameWindow();
	},
	getFrameId: function () {
		return this.FrameId;
	},
	getEditorType: function () {
		return this.EditorType;
	},
	getEditorTransaction: function () {
		return this.EditorTransaction;
	},
	getEditorDocumentId: function () {
		return this.EditorDocumentId;
	},
	getEditorEditorTable: function () {
		return this.EditorEditorTable;
	},
	getEditorIsHot: function () {
		return this.EditorIsHot;
	},
	getEditorEditCmd: function () {
		return this.EditorEditCmd;
	},
	getEditorUrl: function () {
		return this.EditorUrl;
	},
	getEditorDocumentParameters: function () {
		return this.EditorDocumentParameters;
	},
	getEditorMakeNewDoc: function () {
		return this.EditorMakeNewDoc;
	},
	getEditorPublishWhenSave: function () {
		return this.EditorPublishWhenSave;
	},
	getEditorAutoRebuild: function () {
		return this.EditorAutoRebuild;
	},
	getEditorMakeSameDoc: function () {
		return this.EditorMakeSameDoc;
	},
	getEditorDidSetHiddenText: function () {
		return this.EditorDidSetHiddenText;
	},
	getEditorDocumentPath: function () {
		return this.EditorDocumentPath;
	},
	getEditorDocumentText: function () {
		return this.EditorDocumentText;
	},
	getEditorEditPageNr: function () {
		return this.EditorEditPageNr;
	},
	getEditorContentType: function () {
		return this.EditorContentType;
	},
	getEditorIsActive: function () {
		return this.EditorIsActive;
	},
	getEditorIsInUse: function () {
		return this.EditorIsInUse;
	},
	getEditorReloadNeeded: function () {
		return this.EditorReloadNeeded;
	},
	getEditorReloadAllNeeded: function () {
		return this.EditorReloadAllNeeded;
	},
	// setters
	setEditorMakeNewDoc: function (newVal) {
		this.EditorMakeNewDoc = newVal;
	},
	setEditorPublishWhenSave: function (newVal) {
		this.EditorPublishWhenSave = newVal;
	},
	setEditorAutoRebuild: function (newVal) {
		this.EditorAutoRebuild = newVal;
	},
	setEditorMakeSameDoc: function (newVal) {
		this.EditorMakeSameDoc = newVal;
	},
	setEditorDidSetHiddenText: function (newVal) {
		this.EditorDidSetHiddenText = newVal;
	},
	setEditorIsActive: function (newVal) {
		this.EditorIsActive = newVal;
		if (!newVal) {
			return;
		}
		var _theEditorFrame = this.getEditorFrameWindow();
		if (this.getEditorReloadAllNeeded()) {
			if (this.EditorType === "cockpit") {
				if (_theEditorFrame.saveSettings !== undefined) {
					_theEditorFrame.saveSettings();
				}
				var _href = _theEditorFrame.location.href;
				if (_href.charAt(_href.length - 1) === "#") {
					_href = _href.substr(0, _href.length - 1);
				}
				_theEditorFrame.location.href = _href;
				//_theEditorFrame.location.reload();
			} else {
				if (_theEditorFrame.frames.editHeader) {
					_theEditorFrame.frames.editHeader.location.reload();
				}
				if (this.getContentEditor()) {
					top.we_cmd("reload_editpage");
				}
				if (_theEditorFrame.frames.editFooter) {
					_theEditorFrame.frames.editFooter.location.reload();
				}
			}
			// reload all 3 frames
			this.setEditorReloadAllNeeded(false);
			this.setEditorReloadNeeded(false);
		} else if (this.getEditorReloadNeeded()) {
			if (this.EditorType === "cockpit") {
				_theEditorFrame.location.reload();
			} else {
				top.we_cmd("reload_editpage");
			}
			this.setEditorReloadNeeded(false);
		}
	},
	setEditorEditPageNr: function (newVal) {
		this.EditorEditPageNr = newVal;
	},
	setEditorDocumentId: function (newVal) {
		this.EditorDocumentId = Number(newVal);
	},
	setEditorIsHot: function (newVal) {
		var _update = this.EditorIsHot !== newVal;
		this.EditorIsHot = newVal;
		if (_update) {
			this.updateEditorTab();
		}
	},
	setEditorIsLoading: function (newVal) {
		var _update = this.EditorIsLoading !== newVal;
		this.EditorIsLoading = newVal;
		if (_update) {
			this.updateEditorTab();
		}
	},
	setEditorReloadNeeded: function (newVal) {
		this.EditorReloadNeeded = newVal;
	},
	setEditorReloadAllNeeded: function (newVal) {
		this.EditorReloadAllNeeded = newVal;
	},
	switchToContentEditor: function (toContentEditor) {
		var iframe = this.getEditorFrameWindow().document.getElementsByTagName("IFRAME");
		iframe[(toContentEditor ? 2 : 1)].parentElement.style.display = 'block';
		iframe[(toContentEditor ? 1 : 2)].parentElement.style.display = 'none';
	},
	getContentEditorHeightForFrameNr: function (nr) {
		var framesets = this.getEditorFrameWindow().document.getElementsByTagName("FRAMESET");
		if (framesets.length) {
			//FIXME: remove if frames obsolete
			var frameset = framesets[0];
			if (!frameset) {
				return null;
			}
			var rows = frameset.rows;
			if (!rows) {
				return null;
			}
			var parts = rows.split(",");
			return parts[nr];
		}
		var iframes = this.getEditorFrameWindow().document.getElementsByTagName("IFRAME");
		//note embedded elements such as cockpit don't have a
		return (iframes[nr] && iframes[nr].parentElement.style.display === "none" ? "0" : "+1");
	},
	getContentEditor: function () {//iframes are frames in dom too
		if (this.getContentEditorHeightForFrameNr(1) === "0") {
			return this.getEditorFrameWindow().frames[2];
		}
		if (this.getContentEditorHeightForFrameNr(2) === "0") {
			return this.getEditorFrameWindow().frames[1];
		}
		return null;
	},
	getContentFrame: function () {
		return this.getEditorFrameWindow().frames[2];
	}

};
