/* global WE, errorHandler, top */

/**
 * webEdition CMS
 *
 * webEdition CMS
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

WE().layout.weSuggest = {
	/*config for init jquery plugin*/
	config: {
		delay: 150,
		minLength: 2,
		autoFocus: true,
		source: function (request, response) {
			var el = this.element[0];
			var term = request.term;
			if (term in el.cache) {
				response(el.cache[ term ]);
				return;
			}
			var target = WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=selectorSuggest" +
				"&we_cmd[table]=" + el.getAttribute('data-table') +
				"&we_cmd[contenttypes]=" + el.getAttribute('data-contenttype') +
				"&we_cmd[basedir]=" + el.getAttribute('data-basedir') +
				"&we_cmd[max]=" + el.getAttribute('data-max') +
				"&we_cmd[currentDocumentType]=" + el.getAttribute('data-currentDocumentType') +
				"&we_cmd[currentDocumentID]=" + el.getAttribute('data-currentDocumentID') +
				"&we_cmd[query]=" + request.term;
			$.getJSON(target, request, function (weResponse, status, xhr) {
				el.cache[term] = weResponse.DataArray.suggest;
				response(weResponse.DataArray.suggest);
			});
		},
		/*fired on create of widget*/
		create: function () {
			var res = this.getAttribute('data-result');
			this.result = this.ownerDocument.getElementById(res);
			this.cache = {};
		},
		/*called if search via rpc is done*/
		search: function (event, ui) {
			//FIXME result value -1
			this.result.value = 0;
		},
		/*called if an item was selected from suggest*/
		select: function (event, ui) {
			this.result.value = ui.item.ID;
			this.result.setAttribute('data-contenttype', ui.item.contenttype);
			this.classList.remove("weMarkInputError");

			var acID = this.id.search('yuiAcInput') === 0 ? this.id.substr(10) : '';
			var current = {//currentDir : 0,
				currentID: ui.item.ID,
				currentPath: ui.item.value,
				currentType: ui.item.contenttype,
				currentTable: this.getAttribute('data-table')
			};
			event.target.ownerDocument.defaultView.we_cmd('we_suggest_postprocessSelection', current, acID);
		},
		/*called if the element was modified*/
		change: function (event, ui) {
			//set path to / if no path is given
			if (this.value === "" && !this.getAttribute("required")) {//is this correct?!
				this.value = "/";
				this.result.value = 0;
				this.result.setAttribute('data-contenttype', WE().consts.contentTypes.FOLDER_COMMON);
			}
			if (!this.getAttribute("disabled") && this.offsetParent !== null /*returns null if parent is hidden*/ && (
				this.value && this.value !== "/" && !parseInt(this.result.value) || //sth. was typed, but not selected
				!parseInt(this.result.value) && this.getAttribute("required") || //a required field has no value
				(this.getAttribute("data-selector") !== "customerSelector" && this.value && this.value.indexOf(this.getAttribute("data-basedir")) !== 0) || //basedir must match the selected path
				(this.getAttribute("data-selector") === "docSelector" && this.result.getAttribute('data-contenttype') === WE().consts.contentTypes.FOLDER_COMMON) //we need a document, but only a folder is selected
				)
				) {
				this.classList.add("weMarkInputError");
			} else {
				this.classList.remove("weMarkInputError");
			}
		}
	},
	/*if an input element is reused, you can update its config - see source function
	 * possible elements: required, table,contenttypes,basedir,max
	 * */
	updateSelectorConfig: function (win, id, config) {
		var el = win.document.getElementById(id);
		if (!el) {
			WE().t_e('weSuggest unable to update config of ' + id);
			return;
		}
		if (config.required !== undefined) {
			if (config.required) {
				el.setAttribute("required", "required");
			} else {
				el.removeAttribute("required");
			}
			delete config.required;
		}
		for (var c in config) {
			el.setAttribute("data-" + c, config[c]);
		}
	},
	/*ported from old yahoo - removed?*/
	openSelectionToEdit: function (win, elID) {
		var el = win.document.getElementById(elID);
		var table = el.getAttribute('data-table'),
			id = el.result.value,
			type = el.getAttribute('data-contenttype');

		if (table && id && type) {
			WE().layout.openToEdit(table, id, type);
		}
	},
	/*Check if all required fields are set*/
	checkRequired: function (win, id) {
		var isValid = true;
		win.$((id === undefined || !id ? '.weSuggest' : '#' + id)).each(function () {

			if (this.getAttribute("data-result") && this.result) {
				if (!this.getAttribute("disabled") && this.offsetParent !== null /*returns null if parent is hidden*/ && (
					(this.value && this.value !== "/" && !parseInt(this.result.value)) || //sth. was typed, but not selected
					(!parseInt(this.result.value) && this.getAttribute("required")) || //a required field has no value
					(this.getAttribute("data-selector") !== "customerSelector" && this.value && this.value.indexOf(this.getAttribute("data-basedir")) !== 0) || //basedir must match the selected path
					(this.getAttribute("data-selector") === "docSelector" && [WE().consts.contentTypes.FOLDER_COMMON, WE().consts.contentTypes.FOLDER_DOCUMENT, WE().consts.contentTypes.FOLDER_OBJECTFILE,
						WE().consts.contentTypes.FOLDER_TEMPLATE, WE().consts.contentTypes.FOLDER_VFILE
					].indexOf(this.result.getAttribute('data-contenttype')) !== -1 && //we need a document, but only a folder is selected
						!(this.getAttribute("data-table") === WE().consts.tables.OBJECT_FILES_TABLE && this.value === this.getAttribute("data-basedir"))//in case of an object, a selected classfolder is ignored
						)
					)
					) {
					this.classList.add("weMarkInputError");
					isValid = false;
				} else {
					this.classList.remove("weMarkInputError");
				}
			} else // we have no suggestor or no hidden-result-field, e.g. selectors on tag-wizzard
			if (!this.getAttribute("disabled") && this.offsetParent !== null &&
				this.getAttribute("required") && (!parseInt(this.result.value) || this.value === "/")) {
				this.classList.add("weMarkInputError");
				isValid = false;
			} else {
				this.classList.remove("weMarkInputError");
			}
		});
		return isValid;
	}
};