/**
 * source.html
 *
 * Copyright 2013-2014 Web Power, www.webpower.nl
 * @author Arjan Haverkamp
 */
'use strict';

var SourceDialog = {
	tinymcePlugin: window.tinyMCEPopup,
	tinymce: window.tinymce,
	editor: window.tinyMCEPopup.editor,

	codemirror: null,
	chr: 0, // Unused utf-8 character, placeholder for cursor
	isMac: /macintosh|mac os/i.test(navigator.userAgent),
	cmSettings: {
		config: {
			mode: 'application/x-httpd-php',
			//mode: 'htmlmixed',
			theme: 'default',
			lineNumbers: true,
			lineWrapping: true,
			indentUnit: 2,
			tabSize: 2,
			indentWithTabs: true,
			matchBrackets: true,
			styleActiveLine: true
		},
		indentOnInit: true
	},
	saveCursorPosition: true,

	inArray: function (key, arr) {
		arr = '|' + arr.join('|') + '|';
		return arr.indexOf('|' + key + '|') !== -1;
	},

	init: function () {// Initialize (before load)
		var i, userSettings = this.editor.settings.codemirror ? this.editor.settings.codemirror : {};

		if (this.saveCursorPosition) {
			this.editor.selection.setContent('<span style="display: none;" id="mceWesourceCaret"></span>' + this.editor.selection.getContent());
		}

		/*
		 if (userSettings.fullscreen) {
		 this.cmSettings.jsFiles.push('addon/display/fullscreen.js');
		 this.cmSettings.cssFiles.push('addon/display/fullscreen.css');
		 }
		 */

		// Merge config
		for (i in userSettings.config) {
			this.cmSettings.config[i] = userSettings.config[i];
		}

		if (this.cmSettings.config.theme) {
			document.documentElement.className += this.cmSettings.config.theme.replace(/(^|\s)\s*/g, " cm-s-");
		}

		this.start();
	},

	start: function () {// FIXME: move inside init()
		var indentOnInit = this.cmSettings.indentOnInit;

		// Create legend for keyboard shortcuts for find & replace:
		/*
		 var head = parent.document.querySelectorAll('.mce-foot')[0],
		 div = parent.document.createElement('div'),
		 td1 = '<td style="font-size:11px;background:#777;color:#fff;padding:0 4px">',
		 td2 = '<td style="font-size:11px;padding-right:5px">';
		 div.innerHTML = '<table cellspacing="0" cellpadding="0" style="border-spacing:4px"><tr>' + td1 + (isMac ? '&#8984;-F' : 'Ctrl-F</td>') + td2 + tinymce.translate('Start search') + '</td>' + td1 + (isMac ? '&#8984;-G' : 'Ctrl-G') + '</td>' + td2 + tinymce.translate('Find next') + '</td>' + td1 + (isMac ? '&#8984;-Alt-F' : 'Shift-Ctrl-F') + '</td>' + td2 + tinymce.translate('Find previous') + '</td></tr>' + '<tr>' + td1 + (isMac ? '&#8984;-Alt-F' : 'Shift-Ctrl-F') + '</td>' + td2 + tinymce.translate('Replace') + '</td>' + td1 + (isMac ? 'Shift-&#8984;-Alt-F' : 'Shift-Ctrl-R') +'</td>' + td2 + tinymce.translate('Replace all') + '</td></tr></table>';
		 div.style.position = 'absolute';
		 div.style.left = div.style.bottom = '5px';
		 head.appendChild(div);
		 *
		 */

		// Set CodeMirror cursor and bookmark to same position as cursor was in TinyMCE:
		var html = this.editor.getContent({source_view: true});
		// [FIX] #6 z-index issue with table panel and source code dialog
		//	editor.selection.getBookmark();

		html = html.replace(/<span\s+id="mceWesourceCaret"\sstyle="display: none;"([^>]*)><\/span>/gm, String.fromCharCode(this.chr));
		this.editor.dom.remove(this.editor.dom.select('span#mceWesourceCaret'));

		// Hide TinyMCE toolbar panels, [FIX] #6 z-index issue with table panel and source code dialog
		// https://github.com/christiaan/tinymce-codemirror/issues/6
		this.tinymce.each(this.editor.contextToolbars, function (toolbar) {
			if (toolbar.panel) {
				toolbar.panel.hide();
			}
		});

		window.CodeMirror.defineInitHook(function (inst) {
			// Move cursor to correct position:
			inst.focus();
			var cursor = inst.getSearchCursor(String.fromCharCode(this.chr), false);
			if (cursor.findNext()) {
				inst.setCursor(cursor.to());
				cursor.replace('');
			}

			// Indent all code, if so requested:
			if (indentOnInit) {
				var last = inst.lineCount();
				inst.operation(function () {
					for (var i = 0; i < last; ++i) {
						inst.indentLine(i);
					}
				});
			}

		});

		this.cmSettings.config.value = html;
		// Instantiante CodeMirror:
		this.codemirror = window.CodeMirror(document.getElementById('wesourceDiv'), this.cmSettings.config);
		this.codemirror.isDirty = false;
		this.codemirror.on('change', function (inst) {
			inst.isDirty = true;
		});
	},

	findDepth: function (haystack, needle) {
		var idx = haystack.indexOf(needle), depth = 0, x;
		for (x = idx - 1; x >= 0; x--) {
			switch (haystack.charAt(x)) {
				case '<':
					depth--;
					break;
				case '>':
					depth++;
					break;
				case '&':
					depth++;
					break;
			}
		}
		return depth;
	},

	writeBack: function () {
		var cc = '&#x0;';
		var isDirty = this.codemirror.isDirty;
		var doc = this.codemirror.doc;
		var content;

		if (doc.somethingSelected()) {
			doc.setCursor(doc.getCursor());
		}
		doc.replaceSelection(cc);

		var pos = this.codemirror.getCursor(),
			curLineHTML = doc.getLine(pos.line);

		if (this.findDepth(curLineHTML, cc) !== 0) { // FIXME: move cc instead of deleting it
			curLineHTML = curLineHTML.replace(cc, '');
			doc.replaceRange(curLineHTML, window.CodeMirror.Pos(pos.line, 0), window.CodeMirror.Pos(pos.line));
		}

		// [FIX] Cursor position inside JS, style or &nbps;
		// Workaround to fix cursor position if inside script tag
		var code = this.codemirror.getValue();
		if (code.search(/<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/gi) !== -1 || code.search(/<style\b[^<]*(?:(?!<\/style>)<[^<]*)*<\/style>/gi) !== -1) {
			this.editor.setContent(this.codemirror.getValue().replace(cc, ''));
		} else {
			content = this.codemirror.getValue().replace(cc, '<span class="mceWesourceCaret">' + cc + '</span>');
		}

		var editor = this.editor;
		this.editor.undoManager.transact(function () {
			editor.setContent(content);
		});
		this.editor.focus();

		// Set cursor:
		var el = this.editor.dom.select('.mceWesourceCaret')[0];
		if (el) {
			this.editor.selection.scrollIntoView(el);
			this.editor.selection.setCursorLocation(el, 0);
			this.editor.dom.remove(el);
		}
	}
};

function keyDownReplacement(e) {
	switch (e.key) {
		case 'Esc':
			WE().util.dialogTop(window).doClose();
			break;
		case 'Enter':
			e.stopPropagation();
			// do nothing!
			break;
	}
}

function weTinyDialog_doOk() {
	SourceDialog.writeBack();
	WE().util.dialogTop(window).doClose();
}

$(function () {
	SourceDialog.init();
});