<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0627c1498cde $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 12:07 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//start autoloader!
define('WE_DOCUMENT_ROOT', (string) realpath(__DIR__ . '/../'));
require_once('we/include/we.inc.php');

$cmd = we_base_request::string('we_cmd', '', 0);
if($cmd === 'OK'){
	//we need this for htaccess checks
	echo 'OK';
	exit();
}
we_html_tools::protect([], ($cmd === 'we_gui_loggingOut' ? WEBEDITION_DIR . 'index.php' : ''));

if(!$cmd){
	t_e('call without command, might be an error');
	exit();
}

/**
 *
 * @param string $cmd
 * @return boolean|string
 */
function findInclude(string $cmd): bool{
	switch($cmd){
		case 'we_selector_delete':
			if(isset($_SESSION['weS']['seemForOpenDelSelector']['Table'])){
				unset($_SESSION['weS']['seemForOpenDelSelector']['Table']);
			}
		//no break
		case 'we_selector_directory':
		case 'we_selector_document':
		case 'we_selector_file':
		case 'we_selector_image':
		case 'we_users_selector':
		case 'we_voting_dirSelector':
		case 'we_customer_selector':
		case 'we_export_dirSelector':
		case 'we_navigation_dirSelector':
		case 'we_newsletter_dirSelector':
			we_selector_file::getSelectorFromRequest();
			return true;
		case 'we_fileupload_editor':
			we_fileupload_ui_editor::showFrameset();
			return true;
		case 'import_files':
			$what = we_base_request::string('pnt', 'wizframeset');
			$step = we_base_request::int('step', 0);
			$mode = we_base_request::int('mode', 0);
			$import_object = new we_import_files($what, $step, $mode);
			echo $import_object->getHTML();
			return true;
		case 'loadSidebarDocument':
			echo we_sidebar_frames::getHTMLContent();
			return true;
		case 'siteImport':
		case 'SaveWePageSettings' :
		case 'CreateWePageSettings' :
		case 'updateSiteImportTable' :
//FIXME:remove these calls
			$import_object = new we_import_site();
			echo $import_object->getHTML();
			return true;
		case 'loadTree':
			we_export_tree::loadTree();
			return true;
		case 'exit_delete':
		case 'exit_move':
		case 'home':
		case 'reset_home':
		case 'open_cockpit':
			we_gui_cockpit::getEditor();
			return true;
		case 'openColorChooser':
			we_dialog_colorChooser::show();
			return true;
		case 'rebuild':
			$wiz = new we_rebuild_wizard();
			$wiz->showFrameset();
			return true;
		case 'help':
			return true;
		case 'editNewCollection':
			echo we_dialog_newCollection::getDialog();
			return true;
		case 'editMetadataFields':
			echo we_dialog_metadata::getDialog(); //'edit_metadatafields.inc.php';
			return true;
		case 'show':
			require(WE_INCLUDES_PATH . 'we_showDocument.inc.php');
			return true;
		case 'open_url_in_editor': // Beim ungewollten Verlassen (Klick auf Link im Bearbeitenmodus) des Editors wird die Location auf diese Seite weitergeleitet. Hier wird dann ein Kommando gebildet
			echo we_html_tools::getHtmlTop('', we_gui_SEEM::getJavaScriptCommandForOneLink('<a href="' . we_base_request::_(we_base_request::URL, 'we_cmd', '', 1) . '"></a>'), we_html_element::body());
			return true;
		case 'open_form_in_editor': // Formular wird an dieses Skript umgeleitet, hier wird ein Kommando daraus gebaut, um das Dokument korrekt zu öffnen
			echo we_gui_SEEM::openFormInEditor();
			return true;
		case 'showThumbTable':
			$uniqid = we_base_request::string('u');
			$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 't', '');
			$thumbIDs = we_base_request::intList('id', []);
			we_base_showFile::showThumbTable($uniqid, $we_transaction, $thumbIDs);
			return true;
		case 'openBrowser':
			$id = we_base_request::int('id');
			$url = we_base_request::_(we_base_request::URL, 'url', '');
			we_base_showFile::openBrowser($id, $url);
			return true;
		case 'open_extern_document'; // wird benötigt um ein externes Dokument aufzurufen
			$parastr = we_base_request::_(we_base_request::RAW_CHECKED, 'we_cmd', '', 4);
			echo we_gui_SEEM::getExtDocFrameset($parastr);
			return true;
		case 'edit_document_with_parameters':
			$parastr = we_base_request::_(we_base_request::RAW_CHECKED, 'we_cmd', '', 4);
			we_editor_editFrameset::show($parastr);
			return true;
		case 'edit_document':
		case 'edit_folder':
		case 'new_document':
		case 'new_folder':
			we_editor_editFrameset::show();
			return true;
		case 'weNaviEditor':
			we_navigation_navigation::naviEditor();
			return true;
		case 'add_cat':
		case 'add_navi':
		case 'copyDocument':
		case 'copyDocumentSelect':
		case 'delete_all_cats':
		case 'delete_all_navi':
		case 'delete_cat':
		case 'delete_navi':
			we_editor_functions::processPropertyCmd($cmd);
			return true;
		case 'doImage_convertGIF':
		case 'doImage_convertJPEG':
		case 'doImage_convertPNG':
		case 'doImage_crop':
		case 'do_add_thumbnails':
		case 'del_thumb':
		case 'resizeImage':
		case 'rotateImage':
			we_editor_functions::processImageCmd($cmd);
			return true;
		case 'unpublish':
			we_editor_functions::doUnpublish();
			return true;
		case 'save_document':
			we_editor_functions::saveDocument();
			return true;
		case 'add_entry_to_list':
		case 'change_link':
		case 'delete_link':
		case 'delete_list':
		case 'doctype_changed':
		case 'down_entry_at_list':
		case 'insert_entry_at_list':
		case 'remove_image':
		case 'template_changed':
		case 'up_entry_at_list':
		case 'update_image':
		case 'wrap_on_off':
			we_editor_functions::processEditorDocumentCmd($cmd);
			return true;
		case 'load_editor':
		case 'publish':
		case 'reload_hot_editpage':
		case 'reload_editpage':
		case 'revert_published':
		case 'switch_edit_page':
		case 'update_file':
		//variants
		case 'insert_variant':
		case 'move_variant_down':
		case 'move_variant_up':
		case 'preview_variant':
		case 'remove_variant':
			we_editor_functions::processEditorCmd($cmd);
			return true;
		case 'edit_link':
		case 'edit_link_at_class':
		case 'edit_link_at_object':
			echo we_dialog_link::getDialog(false);
			return true;
		case 'delete':
			if(we_base_request::bool('we_cmd', false, 1)){
				we_base_delete::getDialog();
			} else {
				we_gui_cockpit::getEditor();
			}
			return true;
		case 'move':
			if(we_base_request::bool('we_cmd', false, 1)){
				we_base_move::getDialog();
			} else {
				we_gui_cockpit::getEditor();
			}
			return true;
		case 'addToCollection':
			if(we_base_request::bool('we_cmd', false, 1)){
				we_dialog_addToCollection::getDialog();
			} else {
				we_gui_cockpit::getEditor();
			}
			return true;
		case 'delete_single_document':
		case 'do_delete':
			we_base_delete::getDialog();
			return true;
		case 'delInfo':
			we_base_delete::getDelInfo();
			return true;
		case 'delFrag':
			we_dialog_deleteProgress::getDialog();
			return true;
		case 'do_move':
		case 'move_single_document':
			we_base_move::getDialog();
			return true;
		case 'moveInfo':
			we_base_move::getMoveInfo();
			return true;
		case 'do_addToCollection':
			we_dialog_addToCollection::getDialog();
			return true;
		case 'browse_server':
			we_html_tools::protect([we_base_perm::WORK_BROWSE_SERVER, we_base_perm::EXIM_IMPORT_SITE, we_base_perm::ADMINISTRATOR]);
			we_dialog_browseServer::getDialog();
			return true;
		case 'selectorBrowse':
			we_html_tools::protect([we_base_perm::WORK_BROWSE_SERVER, we_base_perm::EXIM_IMPORT_SITE, we_base_perm::ADMINISTRATOR]);
			we_dialog_browseServer::getBody();
			return true;
		case 'selectorBrowseCmd':
			we_html_tools::protect([we_base_perm::WORK_BROWSE_SERVER, we_base_perm::EXIM_IMPORT_SITE, we_base_perm::ADMINISTRATOR]);
			we_dialog_browseServer::getCommand();
			return true;
		case 'selectorEdit':
			echo (new we_editor_selectorEdit(null))->show();
			return true;
		case 'make_backup':
			we_backup_wizard::showBackupFrameset();
			return true;
		case 'recover_backup':
			we_backup_wizard::showRecoverFrameset();
			return true;
		case 'import':
			we_import_wizard::getFrameset();
			return true;
		case 'add_thumbnail':
			echo we_dialog_addThumbnail::getDialog();
		case 'image_convertJPEG':
		case 'image_crop':
		case 'image_resize':
		case 'image_rotate':
			$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', 0, 1);

// init document
			$we_doc = we_contents_root::getSessionDocument($we_transaction);
			echo (new we_editor_imageTransform($we_doc, null, $we_transaction))->show();
			return true;
		//  stuff about accessibility/validation
		case 'checkDocument':
			$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction');
			$we_doc = we_contents_root::getSessionDocument($we_transaction);
			echo (new we_editor_checkDocument($we_doc, null, $we_transaction))->show();
			return true;
		case 'customValidationService':
			echo we_dialog_validation::getDialog();
			return true;
		case 'widget_cmd':
			we_gui_cockpit::processCommand();
			return true;
		case 'loadJSConsts':
			echo we_base_jsConstants::process(we_base_request::string('we_cmd', '', 1));
			return true;
		case 'loadSourceEditorConsts':
			echo we_base_srcEditorConsts::process(we_base_request::string('we_cmd', '', 1));
			return true;
//object
		case 'we_object_class_cssAdd':
		case 'we_object_class_cssDel':
		case 'object_changeTempl_ob':
			we_editor_functions::processObjectCmd($cmd);
			return true;
		case 'object_add_workspace':
		case 'object_del_workspace':
			we_editor_functions::processObjectCommonCmd($cmd);
			return true;
		case 'we_object_class_getClassWs':
		case 'restore_defaults':
//	In this file we cant work with WE_OBJECT_MODULE_PATH, because a prefix is already set in : we_cmd.php
			we_editor_functions::processObjectFileCmd($cmd);
			return true;
		case 'object_obj_search':
			$_REQUEST['cmd'] = 'search_submit';
			$weFrame = new we_search_frames();
			$weFrame->process();
			echo $weFrame->getHTML('cmd');
			return true;
		case 'we_object_file_preview':
			require(WE_INCLUDES_PATH . 'we_showDocument.inc.php');
			return true;
		case 'we_object_class_createTemplate':
			echo (new we_editor_createObjectTemplate(new we_contents_template()))->show();
			return true;
		case 'we_object_class_createTemplateCmd':
			we_editor_createObjectTemplate::cmd();
			return true;
		case 'we_object_class_textAreaEdit':
			$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 3);
			$we_doc = we_contents_root::getSessionDocument($we_transaction);
			echo (new we_editor_objectTextarea($we_doc, null, $we_transaction))->show();
			return true;
		case 'we_showMod':
			$mod = we_base_request::string('mod');
			if(!we_modules_info::isActive($mod)){
				return true;
			}
			$what = we_base_request::string('pnt', 'frameset');
			we_modules_frame::processModulesCommand($mod, $what);
			return true;
		case 'editFull_askForUpload':
			echo we_fileupload_ui_editfull::getDialogYesNoCancel(we_base_request::string('we_cmd', '', 1));
			return true;
//search
		case 'we_search_edit':
			$_REQUEST['mod'] = we_modules_info::SEARCH;
			$_REQUEST['pnt'] = 'show_frameset';

			unset($_SESSION['weS']['module_sessions']['search']);

			if(($cmd1 = we_base_request::string('we_cmd', '', 1))){
				$_SESSION['weS']['search']['keyword'] = $cmd1;
				$_REQUEST['keyword'] = $cmd1;
			} else {
				$_REQUEST['keyword'] = '';
			}

			switch((we_base_request::_(we_base_request::TABLE, 'we_cmd', '', 2))){//FIXME: bad to have different types at one query
				case FILE_TABLE:
					$cmd = 'weSearch_new_forDocuments';
					break;
				case TEMPLATES_TABLE:
					$cmd = 'weSearch_new_forTemplates';
					break;
				case OBJECT_FILES_TABLE :
					$cmd = 'weSearch_new_forObjects';
					break;
				case OBJECT_TABLE :
					$cmd = 'weSearch_new_forClasses';
					break;
				default:
					$cmd = 'weSearch_new';
			}
			$_REQUEST['cmd'] = $cmd;

			we_modules_frame::showModulesFrameSet(we_modules_info::SEARCH);
			return true;
	}
	return false;
}

function findModuleInclude(string $cmd): bool{
	//	In we.inc.php all names of the installed modules have already been searched
	//	so we only have to use the array $we_active_integrated_modules
	[$we, $mod] = explode('_', $cmd, 3);
	if($cmd === 'we_' . $mod . '_edit'){
		if(we_modules_info::isActive($mod)){
			$_REQUEST['mod'] = $mod;
			$_REQUEST['pnt'] = 'show_frameset';
			we_modules_frame::showModulesFrameSet($mod);
			return true;
		}
		we_modules_frame::showNotActivatedMsg($mod);
		return true;
	}

	if($we === 'we'){
		//note mod does not essentially mean it has to be an active module
		$class = 'we_' . $mod . '_cmd';
		if(class_exists($class)){
			return $class::we_cmd($cmd);
		}
	}
	return false;
}

if(!findInclude($cmd) && !findModuleInclude($cmd)){
	//	This is ONLY used in the edit-mode of the documents.
	//	This statement prevents the page from being reloaded.
	t_e('error', 'command \'' . $cmd . '\' not known!');
	//exit('command \'' . $cmd . '\' not known!');
	exit();
}
